# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
from MaKaC.PDFinterface.base import Image as PDFImage, FileDummy
from MaKaC.PDFinterface.base import SimpleParagraph

from MaKaC.registration import Registrant
from conference import Conference
from badge import BadgeTemplateItem
from MaKaC.webinterface.common.countries import CountryHolder
from MaKaC.i18n import _
from indico.core.config import Config
from indico.util.date_time import format_date
from indico.util.string import safe_upper
from datetime import datetime


class RegistrantBadge:

    @classmethod
    def getArgumentType(cls):
        return Registrant

class RegistrantCountry(RegistrantBadge):

    @classmethod
    def getValue(cls, reg):
        return CountryHolder().getCountryById(reg.getCountry())


class RegistrantFullName2(RegistrantBadge):
    """
    FullName without Title.
    """
    @classmethod
    def getValue(cls, reg):
        return reg.getFullName(title=False)

class RegistrantFullName3(RegistrantBadge):
    """
    FullName with Title and with the FirstName first.
    """
    @classmethod
    def getValue(cls, reg):
        return reg.getFullName(firstNameFirst=True)

class RegistrantFullName4(RegistrantBadge):
    """
    FullName without Title and with the FirstName first.
    """
    @classmethod
    def getValue(cls, reg):
        return reg.getFullName(title=False, firstNameFirst=True)

class RegistrantFullName5(RegistrantBadge):
    """
    FullName with Title, the FirstName first and uppercase in surname.
    """

    @classmethod
    def getValue(cls, reg):
        res = "%s %s" % (reg.getFirstName(), safe_upper(reg.getFamilyName()))
        res = res.strip()
        if reg.getTitle() != "":
            res = "%s %s" % (reg.getTitle(), res)
        return res


class RegistrantFullName6(RegistrantBadge):
    """
    FullName without Title, the FirstName first and uppercase inthe surname.
    """

    @classmethod
    def getValue(cls, reg):
        res = "%s %s" % (reg.getFirstName(), safe_upper(reg.getFamilyName()))
        res = res.strip()
        return res


class UpperCaseSurname(RegistrantBadge):
    @classmethod
    def getValue(cls,reg):
        return reg.getSurName().upper()


class TitledFirstName(RegistrantBadge):
    @classmethod
    def getValue(cls,reg):
        return "{} {}".format(reg.getTitle(),reg.getFirstName())


def prettyDateRange(start_date, end_date):
    if not start_date or not end_date: raise NameError("Missing Dates error")
    if not type(start_date) is datetime: raise NameError("Start Date type error")
    if not type(end_date) is datetime: raise NameError("End Date type error")
    ret = []
    sd = str(start_date.day)
    sm = start_date.strftime("%b")
    sy = str(start_date.year)
    ed = str(end_date.day)
    em = end_date.strftime("%b")
    ey = str(end_date.year)
    if start_date.strftime("%x") == end_date.strftime("%x"):
        ret = [ sd, sm, sy[2:] ]
    elif sm == em and sy == ey:
        ret = [sd + "-" + ed, sm, sy[2:]]
    elif sm != em and sy == ey:
        ret = [sd, sm, "-", ed, em, ey[2:]]
    else: ret = [ sd, sm, sy[2:], "-", ed, em, ey[2:] ]
    return " ".join(ret)


class ConferenceDates:

    @classmethod
    def getArgumentType(cls):
        return Conference

    @classmethod
    def getValue(cls, conf):
        return prettyDateRange(conf.getAdjustedStartDate(),conf.getAdjustedEndDate())


class BackToBackDates(RegistrantBadge):

    @classmethod
    def getDates(cls, reg):

        ll = len(reg.getSessionList())
        if len(reg.getSessionList())==0:
            return reg.getConference().getStartDate(), reg.getConference().getEndDate()

        adjusted_sDate = adjusted_eDate = None
        for regSession in reg.getSessionList():
            session = regSession.getSession()
            tStart = session.getMinSlotStartDate()
            tEnd = session.getMaxSlotEndDate()

            if (adjusted_sDate is None) or adjusted_sDate>tStart:
                adjusted_sDate = tStart
            if (adjusted_eDate is None) or adjusted_eDate<tStart:
                adjusted_eDate = tStart

        return adjusted_sDate,adjusted_eDate

    @classmethod
    def getValue(cls,reg):
        adjusted_sDate,adjusted_eDate = cls.getDates(reg)
        return prettyDateRange(adjusted_sDate, adjusted_eDate).upper()


class ExpirationDate(BackToBackDates):

    @classmethod
    def getValue(cls, reg):
        adjusted_sDate,adjusted_eDate = cls.getDates(reg)
        return adjusted_eDate.strftime('%d %b %Y').upper()


class ItemAware():
    @classmethod
    def setItem(cls,item):
        cls._item = item


class QRCode(ItemAware):

    @classmethod
    def getArgumentType(cls):
        return Registrant

    @classmethod
    def getValue(cls, reg):
        from reportlab.lib.units import cm
        from MaKaC.PDFinterface.conference import TicketToPDF
        index, badgeTemplate = reg.getBadgeTemplate()
        item = cls._item
        pWidth = badgeTemplate.pixelsToCm(item.getWidth()) * cm
        pHeight = pWidth
        img = TicketToPDF.getQRCodeAsFile(reg)
        p = PDFImage(filename=img.name,width=pWidth,height=pHeight)
        return p


class ContactType(RegistrantBadge):
    """
    FullName without Title.
    """
    @classmethod
    def getValue(cls, reg):
        return reg.getRepresentationType()["contactType"]


class Picture(ItemAware):

    @classmethod
    def getArgumentType(cls):
        return Registrant

    @classmethod
    def getImage(cls,reg):
        return reg.getPicture()

    @classmethod
    def getValue(cls, reg):
        from reportlab.lib.units import cm
        # set picture height like width
        index, badgeTemplate = reg.getBadgeTemplate()
        item = cls._item
        pWidth = badgeTemplate.pixelsToCm(item.getWidth()) * cm
        pHeight = badgeTemplate.pixelsToCm(item.getHeight()) * cm
        text = cls.getImage(reg)
        if(text):
            p = PDFImage(filename=text.getFilePath(),width=pWidth,height=pHeight)
        else:
            p = SimpleParagraph("NO PICTURE")
        return p


class ConferenceLogo(Picture):
    @classmethod
    def getImage(cls,reg):
        return reg.getConference().getLogo()


class Band(ItemAware):

    @classmethod
    def getArgumentType(cls):
        return Registrant

    @classmethod
    def getSuffix(cls,contactType):
        return "stripe.jpg"

    @classmethod
    def getValue(cls, reg):

        from reportlab.lib.units import cm
        import os
        # set picture height like width

        baseDir = Config.getInstance().getImagesDir()
        index, badgeTemplate = reg.getBadgeTemplate()
        item = cls._item
        pWidth = badgeTemplate.pixelsToCm(item.getWidth()) * cm
        pHeight = badgeTemplate.pixelsToCm(item.getHeight()) * cm if item.getHeight() else 1 * cm
        reprType = reg.getRepresentationType()
        if not reprType:
            contactType = 'No type'
        else:
            contactType = reprType["contactType"]

        if not contactType:
            contactType = 'No type'

        from MaKaC.webinterface.common.contact_types import ContactTypesRegistry
        colorName = ContactTypesRegistry.getMapColorName(contactType)
        imgFileName = os.path.join(baseDir,colorName+cls.getSuffix(contactType))
        p = PDFImage(filename=imgFileName,width=pWidth,height=pHeight)

        return p


class BigLetter(Band):
    @classmethod
    def getSuffix(cls,contactType):
        from MaKaC.webinterface.common.contact_types import ContactTypesRegistry
        letter = ContactTypesRegistry.getMapLetter(contactType)
        return "{}bigletter.png".format(letter)


class BadgeDesignConfiguration:
    """ This class has 2 objects:
    -items_actions maps the name of an item to the action that should be taken
    at the time it is drawed.
    -groups organizes the item names into groups. These groups are used for the
    <select> box in the WConfModifBadgeDesign.tpl file.
    """

    """ Dictionary that maps the name of an item to the action that should be taken
    at the time it is drawed.
    An action can be:
      -A method: depending on the class owning the method, a Conference object,
      a Registrant object, or a BadgeTemplateItem object should be passed to the method.
      The method must return a string.
      For example: 'Full Name' : Registrant.getFullName  means that, if a badgeTemplate
      has a 'Full Name' item, each time a badge will be drawed, the Full Name of the
      registrant will be drawed as returned by the method getFullName of the class Registrant.
      -A class: when there is no method already available for what we need, we have
      to write a custom class (see classes above).
      These classes must have 2 methods:
       *it must have a getArgumentType() method, which returns either Conference, Registrant or BadgeTemplateItem.
       Depending on what is returned, we will pass a different object to the getValue() method.
       *it must have a getValue(object) method, to which a Conference instance, a Registrant instance or a
       BadgeTemplateItem instance must be passed, depending on the result of the getArgumentType() method.

      """
    def __init__(self):

            self.items_actions = { "Title":               (_("Title"), Registrant.getTitle),
                               "Full Name":               (_("Full Name"), Registrant.getFullName),
                               "Full Name (w/o title)":   (_("Full Name (w/o title)"), RegistrantFullName2),
                               "Full Name B":             (_("Full Name B"), RegistrantFullName3),
                               "Full Name B (w/o title)": (_("Full Name B (w/o title)"), RegistrantFullName4),
                               "Full Name C":             (_("Full Name C"), RegistrantFullName5),
                               "Full Name C (w/o title)": (_("Full Name C (w/o title)"), RegistrantFullName6),
                               "First Name":              (_("First Name"), Registrant.getFirstName),
                               "Title First Name":        (_("Title First Name"), TitledFirstName),
                               "Surname":                 (_("Surname"), Registrant.getSurName),
                               "SURNAME":                 (_("SURNAME"), UpperCaseSurname),
                               "Position":                (_("Position"), Registrant.getPosition),
                               "Institution":             (_("Institution"), Registrant.getInstitution),
                               "Country":                 (_("Country"), RegistrantCountry),
                               "City":                    (_("City"), Registrant.getCity),
                               "Address":                 (_("Address"), Registrant.getAddress),
                               "Phone":                   (_("Phone"), Registrant.getPhone),
                               "Fax":                     (_("Fax"), Registrant.getFax),
                               "Email":                   (_("Email"), Registrant.getEmail),
                               "Personal homepage":       (_("Personal homepage"), Registrant.getPersonalHomepage),
                               "Amount":                  (_("Amount"), Registrant.getTotal),
                               "Picture":                 (_("Picture"), Picture),
                               "Conference Name":         (_("Conference Name"), Conference.getTitle),
                               "Conference Short Name":   (_("Conference Short Name"), Conference.getShortTitle),
                               "Conference Dates":        (_("Conference Dates"), ConferenceDates),
                               "Back to back dates":      (_("Back to back dates"),BackToBackDates),
                               "Expiration date":         (_("Expiration date"),ExpirationDate),
                               "QR code":                 (_("QR code"),QRCode),
                               "Contact type":            (_("Contact type"),ContactType),
                               "Country represented":     (_("Country represented"),Registrant.getCountryRepresented),
                               "Entity represented":      (_("Entity represented"),Registrant.getEntityRepresented),
                               "Band":                    (_("Band"),Band),
                               "Big letter":              (_("Big letter"),BigLetter),
                               "Conference logo":         (_("Logo"),ConferenceLogo),
                               "Fixed Text":              (_("Fixed Text"), BadgeTemplateItem.getFixedText)
                            }

            """ Dictionary that maps group names to the item names that fall into that group.
            The groups are only used for the <select> box in the WConfModifBadgeDesign.tpl file.
            """
            self.groups = [( _("Registrant Data"), [
                "Title", "Full Name", "Full Name (w/o title)", "Full Name B", "Full Name B (w/o title)", "Full Name C",
                "Full Name C (w/o title)", "First Name", "Title First Name",  "Surname", "SURNAME", "Position",
                "Institution",  "Country", "City", "Address", "Phone", "Fax", "Email", "Personal homepage", "Amount",
                "Picture","Big letter", "Country represented", "Entity represented"]),
                      ( _("Conference Data"), [
                          "Conference Name",  "Conference Short Name", "Conference Dates", "Back to back dates","Expiration date",
                          "Conference logo", "QR code", "Contact type"]),
                      ( _("Fixed Elements"), [ "Fixed Text", "Band"])]
