# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.

import os
from persistent import Persistent
import tempfile

from indico.util.json import loads
from MaKaC.common.Counter import Counter
from indico.core.config import Config
import conference


class EmailTemplateManager(Persistent):
    """ This class is used to manage the email templates
        of a given conference.
        An instance of this class contains the list of templates
        of a conference. This conference is called the owner of the manager.
    """

    def __init__(self, conf):
        """ Class constructor
            conf: the conference who owns this manager
        """
        self.__conf = conf
        self.__templates = {}
        self.__counter = Counter(1)

    def notifyModification(self):
        self._p_changed = 1

    def getTemplateById(self, templateId):
        """
        Returns a EmailTemplate object, given an id
        """
        return self.__templates[templateId]

    def getTemplateData(self, templateId):
        """
        Returns the data (a list with all the information about a template)
        of a template, directly.
        """
        return self.__templates[templateId].getData()

    def getTemplates(self):
        """ Returns a dictionary of (templateId, EmailTemplate) keys and values
        """
        return self.__templates

    def hasTemplate(self, templateId):
        """ Tests if there is a template stored with the given templateId
        """
        return self.__templates.has_key(templateId)

    def getNewTemplateId(self):

        """ Returns a new an unused templateId
        Increments the templateId counter
        """
        return self.__counter.newCount()

    def storeTemplate(self, templateId, templateData):
        """
        Adds a template to the conference.
        templateData is a string produced by converting the object "template" of the save() javascript
        function of WConfModifEmailDesign.tpl into a JSON string.
        The string templateData is a list composed of:
            -The name of the template
            -A dictionary containing the items of the template.
        """
        if self.__templates.has_key(templateId):
            # template already exists
            self.__templates[templateId].setData(loads(templateData))
        else:
            # template does not exist
            self.__templates[templateId] = EmailTemplate(templateId, loads(templateData))
        self.notifyModification()

    def addTemplate(self, templ, templateId):
        if self.__templates.has_key(templateId):
            return None
        else:
            self.__templates[templateId] = templ
            return templ

    def deleteTemplate(self, templateId):
        """ Deletes a template, if it exists (otherwise it does nothing)
        """
        if self.__templates.has_key(templateId):
            del(self.__templates[templateId])
            self.notifyModification()

    def copyTemplate(self, templateId):
        """Duplicates a template"""
        if self.__templates.has_key(templateId):
            srcTempl = self.getTemplateById(templateId)
            destTempl = srcTempl.clone(self)
            tplData = destTempl.getData()
            tplData[0] += " (copy)"
            destTempl.setData(tplData)

    def getOwner(self):
        return self.__conf

    def exportItems(self,filename):
        import json
        exp = dict((k, v.getData()) for (k, v) in self.__templates.iteritems())
        with open(filename, 'w') as outfile:
            j = json.dump(exp,outfile)

    def importItems(self,filename):
        import json
        with open(filename,'r') as infile:
            payload = json.load(infile)

        for k,v in payload.iteritems():
            self.storeTemplate(k,json.dumps(v))

    def processBody(self,body,prefix,add):
        from lxml import etree
        parser = etree.HTMLParser()
        try:
            body = body.encode('utf-8')
        except:
            pass

        from io import BytesIO
        tree = etree.parse(BytesIO(body),parser)
        idx = 0
        for elem in tree.xpath(".//img"):
            oldsrc = elem.attrib['src']
            if add:
                if oldsrc.startswith('/') and not oldsrc.startswith('/'+prefix):
                    oldsrc = '/{}{}'.format(prefix,oldsrc)
            else:
                oldsrc = oldsrc[len(prefix)+1]

            elem.attrib['src'] = oldsrc

        body = etree.tostring(tree)[119:][:-14]
        return body

    def remapImages(self,prefix='indico',add=True):
        templates = conference.ConferenceHolder().getById('default').getEmailTemplateManager().getTemplates()
        for key in templates:
            template = templates[key]
            data = template.getData()
            body = data[1]['body']
            body = self.processBody(body,prefix,add)
            data[1]['body']=body
            template.notifyModification()


class EmailTemplate (Persistent):
    """ This class represents an email template, which
        will be used to send emails.
    """

    def __init__(self, id, templateData):
        """ Class Constructor
            templateData is the templateData string used in the method storeTemplate() of the class
            EmailTemplateManager, transformed to a Python object with the function loads().
            IMPORTANT NOTE: loads() builds an objet with unicode objects inside.
                            if these objects are then concatenated to str objects (for example in an Indico HTML template),
                            this can give problems. In those cases transform the unicode object to str with .encode('utf-8')
            Thus, its structure is a list composed of:
                -The name of the template
                -A dictionary containing items of the template
        """
        #TODO for Email
        self.__id = id
        self.__templateData = templateData
        self.notifyModification()

    def clone(self, templMan, templId=None):
        if templId == None:
            templId = templMan.getNewTemplateId()
        templData = list(self.getData())
        newTempl = EmailTemplate(templId, templData)
        templMan.addTemplate(newTempl, templId)
        templMan.notifyModification()
        return newTempl

    def notifyModification(self):
        self._p_changed = 1

    def getData(self):
        """ Returns the list with all the information of the template.
        Useful so that javascript can analyze it on its own.
        """
        return self.__templateData

    def setData(self, templateData):
        """ Sets the data of the template
        """
        self.__templateData = templateData
        self.notifyModification()

    def getName(self):
        """ Returns the name of the template
        """
        return self.__templateData[0].encode('utf-8')

    def getItem(self, name):
        """ Returns an object of the class EmailTemplateItem
            which corresponds to the item whose name is 'name'
        """
        for key,val in self.__templateData[1].iteritems():
            if key == name:
                return val
        return None

    def parseQuotes(self, templateData):
        # replace characters Quotes for purpose of Json parsing in the form
        result = []
        result.append(templateData[0])
        templateBody = {}
        for key, val in templateData[1].iteritems():
            val = val.replace('&quot;', ';dbquote;')               # double quote
            templateBody[key] = val.replace('&#39;', ';squote;').encode('utf-8')   # single quote

        result.append(templateBody)
        return result
