# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
import codecs
import csv
from abc import abstractmethod, ABCMeta
from datetime import datetime

from openpyxl import Workbook
from openpyxl.writer.excel import save_virtual_workbook

from MaKaC.conference import Link, CategoryManager
from MaKaC.conference import LocalFile
from MaKaC.review import AbstractStatusAccepted, AbstractStatusProposedToAccept
from MaKaC.webinterface import urlHandlers
from MaKaC.webinterface.common.abstractStatusWrapper import AbstractStatusList
from MaKaC.webinterface.common.contribStatusWrapper import ContribStatusList
from indico.core.config import Config
from indico.util.date_time import format_date


class ExcelAbstractGenerator:
    __metaclass__ = ABCMeta

    def __init__(self):
        self._lines=[]
        self._currentLine=[]
        self._writer = None

    @abstractmethod
    def addValue(self, value):
        """Add a new cell value to the current line"""
        pass

    @abstractmethod
    def write(self, value):
        pass

    @abstractmethod
    def addNumberAsString(self, value, excelSpecific):
        """Add a new cell value (which it has to be interpreted like
        a string not like a number) to the current line. Needed just for Excel (not for OpenOffice)."""
        pass

    @abstractmethod
    def newLine(self):
        """Creates a new line for the excel file"""
        pass

    @abstractmethod
    def getExcelContent(self):
        pass

    def excelFormatting(text):
        if text is None:
            text = ""
        if text.strip() != "":
            text = text.replace('\x0d', '')
        return text
    excelFormatting=staticmethod(excelFormatting)


class CsvGenerator(ExcelAbstractGenerator):
    """It helps to create an Excel CSV file. The way to work with this class
    is the following:
        - Use always "addValue" for adding all the cell values to the current
        line, including empty strings.
        - Once the line was completely filled use "newLine" to follow with the
        filling of the next line of the CSV file.
    For instance:
        excelGen=ExcelGenerator()
        for n in range(0,5):
            for i in ["a","b","c","d"]:
                excelGen.addValue("%s-%s"%(n,i))
            excelGen.newLine()
        print excelGen.getExcelContent()

        The CSV output file will be:
        "0-a";"0-b";"0-c";"0-d"
        "1-a";"1-b";"1-c";"1-d"
        "2-a";"2-b";"2-c";"2-d"
        "3-a";"3-b";"3-c";"3-d"
        "4-a";"4-b";"4-c";"4-d"
        """

    def __init__(self):
        super(CsvGenerator,self).__init__()
        self._writer = csv.writer(self, quoting=csv.QUOTE_NONNUMERIC)

    def addValue(self, value):
        """Add a new cell value to the current line"""
        value = CsvGenerator.excelFormatting(value)
        self._currentLine.append(value)

    def write(self, value):
        self._lines.append(value)

    def addNumberAsString(self, value, excelSpecific=True):
        """Add a new cell value (which it has to be interpreted like
        a string not like a number) to the current line. Needed just for Excel (not for OpenOffice)."""
        if excelSpecific and value.strip() != "":
            value = '="%s"' % CsvGenerator.excelFormatting(value)
        self._currentLine.append(value)

    def newLine(self):
        """Creates a new line for the excel file"""
        self._writer.writerow(self._currentLine)
        self._currentLine = []

    def getExcelContent(self):
        if self._currentLine != []:
            self.newLine()
        return codecs.BOM_UTF8 + ''.join(self._lines)



class ExcelGenerator(ExcelAbstractGenerator):

    def __init__(self):
        super(ExcelGenerator,self).__init__()
        self._wb = Workbook()
        self._ws = self._wb.active
        self._rowIndex = 1
        self._colIndex = 'A'

    def addValue(self, value):
        cell = '{}{}'.format(self._colIndex, self._rowIndex)
        value = CsvGenerator.excelFormatting(value)
        self._ws[cell] = value
        self._colIndex = self.increment(self._colIndex)

    def write(self, value):
        pass

    def addNumberAsString(self, value, excelSpecific=True):
        """Add a new cell value (which it has to be interpreted like
        a string not like a number) to the current line. Needed just for Excel (not for OpenOffice)."""
        if excelSpecific and value.strip() != "":
            value = '="%s"' % ExcelGenerator.excelFormatting(value)
        cell = '{}{}'.format(self._colIndex, self._rowIndex)
        self._ws[cell] = value
        self._colIndex = self.increment(self._colIndex)

    def increment(self,str):
        lastLetter = str[-1]
        result = None
        if lastLetter == 'Z':
            if len(str)<2:
                result = 'AA'
            else:
                subStr = self.increment(str[:-1])
                result = subStr+'A'
        else:
            lastLetter = chr(ord(lastLetter)+1)
            result = str[:-1]+lastLetter
        return result

    def newLine(self):
        self._rowIndex += 1
        self._colIndex = 'A'

    def getExcelContent(self):
        result = save_virtual_workbook(self._wb)
        return result


class RegistrantsListToExcel:

    def __init__(self, conf,list=None, display=["Institution", "Phone", "City", "Country"], excelSpecific=True,csv=True,reduced=False):
        self._conf = conf
        self._regForm = conf.getRegistrationForm()
        self._regList = list
        self._display = display
        self._excelSpecific = excelSpecific
        self._csv = csv
        self._reduced = reduced

    def _formatGenericValue(self, fieldInput, value):
        try:
            if not isinstance(value, LocalFile):
                value = fieldInput.getValueDisplay(value)
            else: # if file, use just the filename, not the html link returned by getValueDisplay
                value = str(value).strip()
        except:
            value = str(value).strip()
        return value

    def _isNumberAsString(self, fieldInput, value):
        return fieldInput.getName() == 'Telephone'

    def getExcelFile(self):
        excelGen= CsvGenerator() if self._csv else ExcelGenerator()

        from MaKaC.registration import LabelInput
        from MaKaC.webinterface.common.countries import CountryHolder
        from MaKaC.registration import FileInput

        reducedKeys = None

        for client, reducedKeysByClient in Config.getInstance().getReducedKeys():
            if client in [CategoryManager().getById(c).getName() for c in self._conf.getCategoriesPath()[0]]:
                reducedKeys = reducedKeysByClient

        if not reducedKeys:
            self._reduced = False

        # Header : Personal Data
        pd = self._regForm.getPersonalData()._pdMap

        pdNames = ['title', 'gender', 'surname', 'firstName', 'email', 'position', 'institution', 'address', 'city',
                   'country', 'phone', 'fax',
                   'birthDate', 'personalHomepage', 'passportId', 'passportExpire', 'passportOrigin']

        pdItems = []
        for item in pdNames:
            if not self._reduced or item not in reducedKeys:
                excelGen.addValue(pd[item].getCaption())
                pdItems.append(pd[item].getId())

        # Header : Representation Type
        repf = self._regForm.getRepresentationForm()

        repItems = ['repType', 'repSubType', 'countryRepresentative', 'organizationRepresentative', 'contactType']

        for item in repItems:
            if not self._reduced or item not in reducedKeys:
                excelGen.addValue(repf.getFieldCaption(item))

        # Header : Statuses
        statuses = self._regForm.getStatuses().keys()
        statuses.sort()
        staItems = []
        for item in statuses:
            if not self._reduced or self._regForm.getStatusById(item).getCaption() not in reducedKeys:
                excelGen.addValue('s-'+self._regForm.getStatusById(item).getCaption())
                staItems.append(self._regForm.getStatusById(item).getId())  # id with name : approved, panel ...

        # Header : User sections
        sectIds = self._regForm.getGeneralSectionForms().keys()
        sectIds.sort()
        userItems = []
        for item in sectIds:
            sect = self._regForm.getGeneralSectionFormById(item)
            if sect.isEnabled() and sect.getTitle()!='Personal Data':
                userItems.append(item) # keep only valid sections
                prefix = sect.getTitle()[:2]+'-'
                for fld in sect.getSortedFields():
                    if isinstance(fld.getInput(), LabelInput):  # skip field
                        continue

                    if not self._reduced or fld.getCaption().lower() not in reducedKeys:
                        excelGen.addValue(prefix+fld.getCaption())

        # Header : Accommodation
        accf = self._regForm.getAccommodationForm()
        if accf.isEnabled():
            excelGen.addValue('Accommodation')
            excelGen.addValue('Arrival Date')
            excelGen.addValue('Departure Date')

        # Header : Session : NOT EXPORT
        # sesf = self._regForm.getSessionsForm()
        # if sesf.isEnabled():
        #     sesItems = sesf.getSessions().keys()
        #     sesItems.sort()  # order sessions by id
        #     for item in sesItems:
        #         excelGen.addValue(sesf.getSessions()[item].getCaption())

        # Header : Reason Participation
        reaf = self._regForm.getReasonParticipationForm()
        if reaf.isEnabled():
            excelGen.addValue('Reason Participation')

        # Header : Social Events
        socf = self._regForm.getSocialEventForm()
        if socf.isEnabled():
            socItems = []
            for e in socf.getSocialEventList():  # build list ids
                socItems.append(e.getId())
            socItems.sort()
            for item in socItems:  # order events by id
                excelGen.addValue(socf.getSocialEventById(item).getCaption())

        excelGen.addValue('Registration Date')

        # Data
        for reg in self._regList:
            excelGen.newLine()

            # Data : Personal Data
            grp = reg.getMiscellaneousGroupById(self._regForm.getPersonalData().getId())
            for item in pdItems:
                fld = grp.getResponseItemById(item)
                if not fld:
                    excelGen.addValue('')
                    continue
                name = fld.getName()

                if not self._reduced or name not in reducedKeys:
                    if name in ['birthDate', 'passportExpire']:
                        excelGen.addValue(fld.getValue().strftime("%d/%m/%Y") if fld.getValue() else '')
                    elif name in ['country', 'passportOrigin']:
                        excelGen.addValue(CountryHolder().getCountryById(fld.getValue()))
                    else:
                        excelGen.addValue(fld.getValue())

            # Data : Representation Type
            for item in repItems:
                if not self._reduced or item not in reducedKeys:
                    excelGen.addValue(reg.getRepresentationType()[item])

            # Data : Statuses
            for item in staItems:
                status = reg.getStatusById(item)
                if not self._reduced or status.getStatusValue().getCaption().lower() not in reducedKeys:
                    excelGen.addValue(status.getStatusValue().getCaption() if status else '')

            # Data : User sections
            for item in userItems:
                grp = reg.getMiscellaneousGroupById(item)
                for f in self._regForm.getGeneralSectionFormById(item).getSortedFields(): # keep same order as header
                    if not self._reduced or f.getCaption().lower() not in reducedKeys:
                        if isinstance(f.getInput(), LabelInput):  # skip field
                            continue
                        if not grp: # grp not in registration
                            excelGen.addValue('')
                            continue
                        infoItem = grp.getResponseItemById(f.getId())
                        if not(infoItem):  # field not in registration
                            excelGen.addValue('')
                            continue
                        if isinstance(infoItem.getGeneralField().getInput(), FileInput):
                            excelGen.addValue('Yes' if infoItem.getValue() else 'No')
                        else:
                            excelGen.addValue(infoItem.getValue())

            # Data : Accommodation
            if accf.isEnabled():
                ac = reg.getAccommodation()
                if ac:
                    excelGen.addValue(ac.getAccommodationType().getCaption())
                    excelGen.addValue(ac.getArrivalDate().strftime("%d/%m/%Y"))
                    excelGen.addValue(ac.getDepartureDate().strftime("%d/%m/%Y"))
                else:
                    excelGen.addValue('')
                    excelGen.addValue('')
                    excelGen.addValue('')

            # Data : Session : NOT EXPORT

            # Data : Reason Participation
            if reaf.isEnabled():
                excelGen.addValue(reg.getReasonParticipation())

            # Data : Social Events
            if socf.isEnabled():
                for item in socItems:
                    excelGen.addValue('Yes' if reg.getSocialEventById(item) else 'No')

            excelGen.addValue(reg.getRegistrationDate().strftime("%d/%m/%Y"))

        return excelGen.getExcelContent()


class AbstractListToExcel:

    def __init__(self, conf, abstractList=None, display=["ID", "Title", "PrimaryAuthor", "Tracks", "Type", "Status", "Rating", "AccTrack", "AccType", "SubmissionDate", "ModificationDate"], excelSpecific=True):
        self._conf = conf
        self._abstractList = abstractList
        self._displayList = display
        self._excelSpecific = excelSpecific

    def getExcelFile(self):
        excelGen = ExcelGenerator()
        for key in self._displayList:
            excelGen.addValue(key)

        excelGen.newLine()

        if self._abstractList is None:
            self._abstractList = self._conf.getAbstractMgr().getAbstractList()

        for abstract in self._abstractList:
            for key in self._displayList:
                status = abstract.getCurrentStatus()
                isStatusAccpeted = isinstance(status, (AbstractStatusAccepted, AbstractStatusProposedToAccept))
                if key == "ID":
                    excelGen.addValue(abstract.getId())
                elif key == "Title":
                    excelGen.addValue(abstract.getTitle())
                elif key == "PrimaryAuthor":
                    paList = []
                    for pa in abstract.getPrimaryAuthorList():
                        paList.append(pa.getFullName())
                    excelGen.addValue("; ".join(paList))
                elif key == "Tracks":
                    trList = []
                    for tr in abstract.getTrackList():
                        trList.append(tr.getTitle())
                    excelGen.addValue("; ".join(trList))
                elif key == "Type":
                    contribType = abstract.getContribType()
                    ctname = ""
                    if contribType is not None:
                        ctname = contribType.getName()
                    excelGen.addValue(ctname)
                elif key == "Status":
                    excelGen.addValue(AbstractStatusList.getInstance().getCaption(status.__class__))
                elif key == "Rating":
                    if abstract.getRating():
                        excelGen.addValue("%.2f" % abstract.getRating())
                    else:
                        excelGen.addNumberAsString("-", self._excelSpecific)
                elif key == "AccTrack":
                    accTrack = ""
                    if isStatusAccpeted and status.getTrack():
                        accTrack = status.getTrack().getCode()
                    excelGen.addValue(accTrack)
                elif key == "AccType":
                    accType = ""
                    if isStatusAccpeted and status.getType():
                        accType = status.getType().getName()
                    excelGen.addValue(accType)
                elif key == "SubmissionDate":
                    excelGen.addValue(format_date(abstract.getSubmissionDate(), format="short"))
                elif key == "ModificationDate":
                    excelGen.addValue(format_date(abstract.getModificationDate(), format="short"))

            excelGen.newLine()
        return excelGen.getExcelContent()

class ParticipantsListToExcel:

    def __init__(self, conf,list=None, excelSpecific=True):
        self._conf = conf
        self._partList = list
        self._excelSpecific = excelSpecific

    def getExcelFile(self):
        excelGen=ExcelGenerator()
        excelGen.addValue("Name")
        excelGen.addValue("Email")
        excelGen.addValue("Affiliation")
        excelGen.addValue("Address")
        excelGen.addNumberAsString("Phone", self._excelSpecific)
        excelGen.addNumberAsString("Fax", self._excelSpecific)
        excelGen.newLine()

        if self._partList == None:
            self._partList = self._conf.getParticipation().getParticipantList()

        for reg in self._partList:
            excelGen.addValue(reg.getFullName())
            excelGen.addValue(reg.getEmail())
            excelGen.addValue(reg.getAffiliation())
            excelGen.addValue(reg.getAddress())
            excelGen.addNumberAsString(reg.getTelephone(), self._excelSpecific)
            excelGen.addNumberAsString(reg.getFax(), self._excelSpecific)
            excelGen.newLine()
        return excelGen.getExcelContent()



class ContributionsListToExcel:

    def __init__(self, conf,contribList=None, tz=None):
        self._conf = conf
        self._contribList = contribList
        if not tz:
            self._tz = self._conf.getTimezone()
        else:
            self._tz = tz

    def getExcelFile(self):
        excelGen=ExcelGenerator()
        excelGen.addValue("Id")
        excelGen.addValue("Date")
        excelGen.addValue("Duration")
        excelGen.addValue("Type")
        excelGen.addValue("Title")
        excelGen.addValue("Presenter")
        excelGen.addValue("Session")
        excelGen.addValue("Track")
        excelGen.addValue("Status")
        excelGen.addValue("Material")
        excelGen.newLine()

        for contrib in self._contribList:
            excelGen.addValue(contrib.getId())
            startDate = contrib.getAdjustedStartDate(self._tz)
            if startDate:
                excelGen.addValue(startDate.strftime("%A %d %b %Y at %H:%M"))
            else:
                excelGen.addValue("")
            excelGen.addValue((datetime(1900,1,1)+contrib.getDuration()).strftime("%Hh%M'"))
            type = contrib.getType()
            if type:
                excelGen.addValue(type.getName())
            else:
                excelGen.addValue("")
            excelGen.addValue(contrib.getTitle())
            listSpeaker = []
            for speaker in contrib.getSpeakerList():
                listSpeaker.append(speaker.getFullName())
            excelGen.addValue("\n".join(listSpeaker))
            session = contrib.getSession()
            if session:
                excelGen.addValue(session.getTitle())
            else:
                excelGen.addValue("")
            track = contrib.getTrack()
            if track:
                excelGen.addValue(track.getTitle())
            else:
                excelGen.addValue("")
            status=contrib.getCurrentStatus()
            excelGen.addValue(ContribStatusList().getCaption(status.__class__))
            listResource = []
            for material in contrib.getAllMaterialList():
                for resource in material.getResourceList():
                    if isinstance(resource, Link):
                        listResource.append(resource.getURL())
                    else:
                        listResource.append(str(urlHandlers.UHFileAccess.getURL( resource )))
            excelGen.addValue("\n".join(listResource))
            excelGen.newLine()

        return excelGen.getExcelContent()
