# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
import MaKaC.common.filters as filters
from MaKaC.webinterface.common.countries import CountryHolder

# -------------- FILTERING ------------------
class AccommFilterField( filters.FilterField ):
    """Contains the filtering criteria for the track of a contribution.

        Inherits from: AbstractFilterField

        Attributes:
            _values -- (list) List of track identifiers;
            _showNoValue -- (bool) Tells whether an contribution satisfies the
                filter if it hasn't belonged to any track.
    """

    _id = "accomm"

    def satisfies( self, reg ):
        """
        """
        accomTypesList = reg.getRegistrationForm().getAccommodationForm().getAccommodationTypesList()
        if len(accomTypesList) != len(self._values):
            if reg.getAccommodation().getAccommodationType() is None:
                return self._showNoValue
            if reg.getAccommodation().getAccommodationType() not in accomTypesList:
                return self._showNoValue
            return reg.getAccommodation().getAccommodationType().getId() in self._values
        else:
            return True


class SessionFilterField( filters.FilterField ):
    """
    """
    _id = "session"

    def satisfies( self, reg ):
        """
        """
        if len(reg.getRegistrationForm().getSessionsForm().getSessions().values()) != len(self._values):
            if reg.getSessionList():
                for sess in reg.getSessionList():
                    if sess.getId() in self._values:
                        return True
                    elif sess.getRegSession() not in reg.getRegistrationForm().getSessionsForm().getSessionList():
                        return self._showNoValue
            else:
                return self._showNoValue
            return False
        else:
            return True

class SessionFirstPriorityFilterField( filters.FilterField ):
    """
    """
    _id = "sessionfirstpriority"

    def satisfies( self, reg ):

        """
        """

        if len(reg.getSessionList()) > 0:
            sess=reg.getSessionList()[0]
            if sess.getId() in self._values:
                return True
            elif sess not in reg.getRegistrationForm().getSessionsForm().getSessionList():
                return self._showNoValue
        else:
            return self._showNoValue
        return False


class EventFilterField(filters.FilterField):
    """
    """
    _id = "event"

    def satisfies(self, reg):
        """
        """
        if (reg.getRegistrationForm().getSocialEventForm().getSocialEventList()) != len(self._values):
            if reg.getSocialEvents():
                for event in reg.getSocialEvents():
                    if event.getId() in self._values:
                        return True
                    elif event.getSocialEventItem() not in reg.getRegistrationForm().getSocialEventForm().getSocialEventList():
                        return self._showNoValue
            else:
                return self._showNoValue
            return False
        return True

class StatusesFilterField(filters.FilterField):
    """
    Filter used for filtering by the statuses (if they exist).
    The statuse's values are represented following this format:
        -NameOfTheStatus + IdOfTheStatus + "-" + IdOfTheValue
        -For "-- not set --" values:
         NameOfTheStatus + IdOfTheStatus + "-NoValue"
    """
    _id = "statuses"

    def __init__(self,conf,values,showNoValue=True ):
        filters.FilterField.__init__(self, conf, values, showNoValue)
        self._confStatusSet = set([status.getCaption()+"-"+status.getId() for status in self._conf.getRegistrationForm().getStatusesList(False)])

    def satisfies(self, reg):
        """
        """
        ### If all the status values are selected, there is no need for filtering
        values = 0
        for status in self._conf.getRegistrationForm().getStatusesList(False):
            values += len(status.getStatusValues()) + 1
        if len(self._values) == values:
            return True

        statusDict = reg.getStatuses()

        if statusDict:
            for status in statusDict:
                if statusDict[status].getCaption()+"-"+statusDict[status].getId() in self._confStatusSet:
                    if statusDict[status] and statusDict[status].getStatusValue():
                        field = statusDict[status].getCaption()+statusDict[status].getId() + "-" +statusDict[status].getStatusValue().getCaption() + ""
                        if field not in self._values:
                            return False
                    else:
                        if statusDict[status].getCaption()+statusDict[status].getId()+"-NoValue" not in self._values:
                            return False
                else:
                    continue
            return True
        else:
            return True

class RegistrantFilterField(filters.FilterField):
    """
    Filter used for filtering the registrants for Registrar linked to a specific affiliation.
    """
    _id = "registrant"

    def satisfies(self, reg):
        """
        OK if registrant affiliation is same as registrar affiliation.
        """
        ### If no value, there is no need for filtering
        if self.getValues()==['']:
            return True
        affiliation = self.getValues()[0]
        rep = reg.getRepresentationType()
        if 'Organisation' in affiliation:
            if affiliation['Organisation'].strip() and (affiliation['Organisation']!=rep['organizationRepresentative']):
                return False
        if 'Representation' in affiliation:
            if not self.matchRepCriteria(affiliation['Representation'], rep['repType']):
                return False
        if 'Country' in affiliation:
            if not self.matchRepCriteria(affiliation['Country'], rep['countryRepresentative']):
                return False
        if 'Sessions' in affiliation:
            if not self.matchSession(affiliation['Sessions'], reg):
                return False
        return True

    def matchRepCriteria(self, critValues, val):
        if not(val) or (val==''):
            return False
        for v in critValues:
            if val==v:
                return True
        return False

    def matchSession(self, critSessions, reg):
        if not reg.getSessionList():
            return False
        for s in critSessions:
            if reg.getSession(s):
                return True
        return False

class RepresentationFilterField(filters.FilterField):
    """
    Filter used for filtering by representation type.
    """
    _id = "representation"

    def satisfies(self, reg):
        if not self._values:
            return True
        reprTypes = reg.getRepresentationType()
        for option in self._values:
            if reprTypes["repType"]==option:
                return True
        return False

class ContactFilterField(filters.FilterField):
    """
    Filter used for filtering by contact type.
    """
    _id = "contact"

    def satisfies(self, reg):
        if not self._values:
            return True
        reprTypes = reg.getRepresentationType()
        for option in self._values:
            if reprTypes["contactType"]==option:
                return True
        return False

class CountryFilterField(filters.FilterField):
    """
    Filter used for filtering by country represented.
    """
    _id = "country"

    def satisfies(self, reg):
        if not self._values:
            return True
        reprTypes = reg.getRepresentationType()
        for option in self._values:
            if reprTypes["countryRepresentative"]==option:
                return True
        return False

class UserSectionFilterField(filters.FilterField):
    """
    Filter used for filtering by the user created fields.
    Format field : fieldId + '-' + fieldName
    """
    _id = "usersection"
    _grpId = None

    def __init__(self, conf, values, name):
        """
        Expect a user section name in format : sectionname-grpId
        """
        filters.FilterField.__init__(self, conf, values)
        self._grpId = name.split('-')[1]

    def satisfies(self, reg):
        if not self._values:
            return True
        regForm = self._conf.getRegistrationForm()
        if not self._grpId or self._grpId not in regForm.getGeneralSectionForms(): # section not defined
            return True
        if self._grpId not in reg.getMiscellaneousGroups(): # reg does not have section
            return False

        regItems = reg.getMiscellaneousGroups()[self._grpId].getResponseItems()
        for option in self._values:
            id = option.split('-')[0]  # format : id-name
            if id in regItems:
                input = regItems[id].getGeneralField().getInput().getId()
                if (input=='checkbox' and regItems[id].getValue()=="yes") \
                    or (input=='file' and regItems[id].getValue()):
                    return True
        return False

class CheckedFilterField(filters.FilterField):
    """
    Filter used for filtering by checked in status.
    """
    _id = "checked"

    def satisfies(self, reg):
        return (reg.isCheckedIn()==True)

class PrintedFilterField(filters.FilterField):
    """
    Filter used for filtering by badge printed.
    """
    _id = "printed"

    def satisfies(self, reg):
        return (reg.getPrintedBadge()>0)


class RegFilterCrit(filters.FilterCriteria):
    _availableFields = { \
        AccommFilterField.getId():AccommFilterField,
        SessionFilterField.getId():SessionFilterField,
        SessionFirstPriorityFilterField.getId():SessionFirstPriorityFilterField,
        EventFilterField.getId():EventFilterField,
        StatusesFilterField.getId():StatusesFilterField,
        RegistrantFilterField.getId():RegistrantFilterField,
        RepresentationFilterField.getId():RepresentationFilterField,
        UserSectionFilterField.getId():UserSectionFilterField,
        ContactFilterField.getId():ContactFilterField,
        CountryFilterField.getId():CountryFilterField,
        CheckedFilterField.getId():CheckedFilterField,
        PrintedFilterField.getId():PrintedFilterField
    }

    def _createField( self, klass, values, name=None ):
        """Contains the creation of filter fields.
        """
        if name:
            return klass(self._conf,values,name)
        else:
            return klass(self._conf,values)

#------------- SORTING --------------------
class RegistrantSortingField(filters.SortingField):

    def getSpecialId(self):
        try:
            if self._specialId:
                pass
        except AttributeError, e:
            return self._id
        return self._specialId

class IdSF(RegistrantSortingField):
    _id="Id"

    def compare( self, r1, r2 ):
        """
        """
        res1 = int(r1.getId())
        res2 = int(r2.getId())
        return cmp( res1, res2 )

class NameSF(RegistrantSortingField):
    _id="Name"

    def compare( self, r1, r2 ):
        return cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName() )

class PositionSF( RegistrantSortingField ):
    _id = "Position"

    def compare( self, r1, r2 ):
        return cmp( r1.getPosition().lower(), r2.getPosition().lower() )

class CountrySF(RegistrantSortingField):
    _id = "Country"

    def compare(self, r1, r2):
        return cmp((CountryHolder().getCountryById(r1.getCountry()) or '').lower(),
                   (CountryHolder().getCountryById(r2.getCountry()) or '').lower())

class CitySF( RegistrantSortingField ):
    _id = "City"

    def compare( self, r1, r2 ):
        return cmp( r1.getCity().lower(), r2.getCity().lower() )

class PhoneSF( RegistrantSortingField ):
    _id = "Phone"

    def compare( self, r1, r2 ):
        return cmp( r1.getPhone().lower(), r2.getPhone().lower() )

class InstitutionSF( RegistrantSortingField ):
    _id = "Institution"

    def compare( self, r1, r2 ):
        return cmp( r1.getInstitution().lower(), r2.getInstitution().lower() )

class EmailSF( RegistrantSortingField ):
    _id = "Email"

    def compare( self, r1, r2 ):
        return cmp( r1.getEmail().lower(), r2.getEmail().lower() )

class SessionsSF( RegistrantSortingField ):
    _id = "Sessions"

    def compare( self, r1, r2 ):
        ses1 = r1.getSessionList()
        ses2 = r2.getSessionList()
        i = 0
        while(i<min(len(ses1), len(ses2))):
            v = cmp( ses1[i].getTitle().lower(), ses2[i].getTitle().lower() )
            if v != 0:
                return v
            i += 1
        if len(ses1)>len(ses2):
            return 1
        elif len(ses1)<len(ses2):
            return -1
        else:
            return 0

class AccommodationSF( RegistrantSortingField ):
    _id = "Accommodation"

    def compare( self, r1, r2 ):
        if r2.getAccommodation() is None and r1.getAccommodation() is not None:
            return 1
        elif r1.getAccommodation() is None and r2.getAccommodation() is not None:
            return -1
        elif r1.getAccommodation() is None and r2.getAccommodation() is None:
            return 0
        elif r2.getAccommodation().getAccommodationType() is None and r1.getAccommodation().getAccommodationType() is not None:
            return 1
        elif r1.getAccommodation().getAccommodationType() is None and r2.getAccommodation().getAccommodationType() is not None:
            return -1
        elif r1.getAccommodation().getAccommodationType() is None and r2.getAccommodation().getAccommodationType() is None:
            return 0
        else:
            return cmp(r1.getAccommodation().getAccommodationType().getCaption(), r2.getAccommodation().getAccommodationType().getCaption())

class ArrivalDateSF( RegistrantSortingField ):
    _id = "ArrivalDate"

    def compare( self, r1, r2 ):
        if r2.getAccommodation() is None and r1.getAccommodation() is not None:
            return 1
        elif r1.getAccommodation() is None and r2.getAccommodation() is not None:
            return -1
        elif r1.getAccommodation() is None and r2.getAccommodation() is None:
            return 0
        elif r2.getAccommodation().getArrivalDate() is None and r1.getAccommodation().getArrivalDate() is not None:
            return 1
        elif r1.getAccommodation().getArrivalDate() is None and r2.getAccommodation().getArrivalDate() is not None:
            return -1
        elif r1.getAccommodation().getArrivalDate() is None and r2.getAccommodation().getArrivalDate() is None:
            return 0
        else:
            return cmp(r1.getAccommodation().getArrivalDate(), r2.getAccommodation().getArrivalDate())

class DepartureDateSF( RegistrantSortingField ):
    _id = "DepartureDate"

    def compare( self, r1, r2 ):
        if r2.getAccommodation() is None and r1.getAccommodation() is not None:
            return 1
        elif r1.getAccommodation() is None and r2.getAccommodation() is not None:
            return -1
        elif r1.getAccommodation() is None and r2.getAccommodation() is None:
            return 0
        elif r2.getAccommodation().getDepartureDate() is None and r1.getAccommodation().getDepartureDate() is not None:
            return 1
        elif r1.getAccommodation().getDepartureDate() is None and r2.getAccommodation().getDepartureDate() is not None:
            return -1
        elif r1.getAccommodation().getDepartureDate() is None and r2.getAccommodation().getDepartureDate() is None:
            return 0
        else:
            return cmp(r1.getAccommodation().getDepartureDate(), r2.getAccommodation().getDepartureDate())

class SocialEventsSF( RegistrantSortingField ):
    _id = "SocialEvents"

    def compare( self, r1, r2 ):
        ses1 = r1.getSocialEvents()
        ses2 = r2.getSocialEvents()
        i = 0
        while(i<min(len(ses1), len(ses2))):
            v = cmp( ses1[i].getCaption().lower(), ses2[i].getCaption().lower() )
            if v != 0:
                return v
            i += 1
        if len(ses1)>len(ses2):
            return 1
        elif len(ses1)<len(ses2):
            return -1
        else:
            return 0

class ReasonParticipationSF( RegistrantSortingField ):
    _id = "ReasonParticipation"

    def compare( self, r1, r2 ):
        rp1=r1.getReasonParticipation() or ""
        rp2=r2.getReasonParticipation() or ""
        return cmp( rp1.lower(), rp2.lower() )

class AddressSF( RegistrantSortingField ):
    _id = "Address"

    def compare( self, r1, r2 ):
        a1=r1.getAddress() or ""
        a2=r2.getAddress() or ""
        return cmp( a.lower().strip(), a.lower().strip() )

class RegistrationDateSF( RegistrantSortingField ):
    _id = "RegistrationDate"

    def compare( self, r1, r2 ):
        rd1=r1.getRegistrationDate() or datetime(1995, 1, 1)
        rd2=r2.getRegistrationDate() or datetime(1995, 1, 1)
        return cmp( rd1, rd2 )


class GeneralFieldSF( RegistrantSortingField ):
    _id = "groupID-fieldId"

    def compare( self, r1, r2 ):
        if self.getSpecialId() != self._id:
            ids=self.getSpecialId().split("-")
            if len(ids)==2:
                group1=r1.getMiscellaneousGroupById(ids[0])
                group2=r2.getMiscellaneousGroupById(ids[0])
                v1=""
                if group1 is not None:
                    i1=group1.getResponseItemById(ids[1])
                    if i1 is not None:
                        v1=i1.getValue()
                v2=""
                if group2 is not None:
                    i2=group2.getResponseItemById(ids[1])
                    if i2 is not None:
                        v2=i2.getValue()
                return cmp(str(v1).lower().strip(), str(v2).lower().strip())
        return 0


class StatusesSF( RegistrantSortingField ):
    _id = "s-statusId"


    def compare( self, r1, r2 ):
        if self.getSpecialId() != self._id:
            ids=self.getSpecialId().split("-")
            if len(ids)==2:
                s1=r1.getStatusById(ids[1])
                s2=r2.getStatusById(ids[1])
                if s1.getStatusValue() is None and s2.getStatusValue() is None:
                    return 0
                if s1.getStatusValue() is None:
                    return -1
                if s2.getStatusValue() is None:
                    return 1
                return cmp( s1.getStatusValue().getCaption().lower().strip(), s2.getStatusValue().getCaption().lower().strip() )
        return 0

class IsPayedSF(RegistrantSortingField):
    _id="isPayed"

    def compare( self, r1, r2 ):
        """
        """
        return cmp( r1.isPayedText(), r2.isPayedText() )

class IdPayment(RegistrantSortingField):
    _id="idpayment"

    def compare( self, r1, r2 ):
        """
        """
        return cmp( r1.getIdPay(), r2.getIdPay() )


class IsCheckedInSF(RegistrantSortingField):
    _id="checkedIn"

    def compare(self, r1, r2):
        """
        """
        return cmp(r1.isCheckedIn(), r2.isCheckedIn())


class CheckInDateSF(RegistrantSortingField):
    _id="checkInDate"

    def compare(self, r1, r2):
        """
        """
        if r2.getCheckInDate() is None and r1.getCheckInDate() is not None:
            return 1
        elif r1.getCheckInDate() is None and r2.getCheckInDate() is not None:
            return -1
        elif r1.getCheckInDate() is None and r2.getCheckInDate() is None:
            return 0
        else:
            return cmp(r1.getCheckInDate(), r2.getCheckInDate())

class ProtocolSF(RegistrantSortingField):
    _id="CountryRepresented"

    def compare(self, r1, r2):
        """
        """
        rep1 = r1.getRepresentationType()
        k1 = rep1["countryRepresentative"] if rep1["repType"]=="Government" else rep1["organizationRepresentative"]

        rep2 = r2.getRepresentationType()
        k2 = rep2["countryRepresentative"] if rep2["repType"]=="Government" else rep2["organizationRepresentative"]

        if k1==k2:
            p1 = rep1["protocol"] if "protocol" in rep1 else 9999
            p2 = rep2["protocol"] if "protocol" in rep2 else 9999
            return cmp(p1,p2) if p1!=p2 else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())
        else:
            return cmp(k1,k2)

class RepresentationTypeSF(RegistrantSortingField):
    _id="RepresentationType"

    def compare( self, r1, r2 ):
        # if same value, order by name
        res = cmp(r1.getRepresentationType()["repType"], r2.getRepresentationType()["repType"])
        return res if res else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())

class RepresentationSubTypeSF(RegistrantSortingField):
    _id="RepresentationSubType"

    def compare( self, r1, r2 ):
        # if same value, order by name
        res = cmp(r1.getRepresentationType()["repSubType"], r2.getRepresentationType()["repSubType"])
        return res if res else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())

class OrganisationRepresentedSF(RegistrantSortingField):
    _id="OrganisationRepresented"

    def compare( self, r1, r2 ):
        # if same value, order by name
        res = cmp(r1.getRepresentationType()["organizationRepresentative"].lstrip(), r2.getRepresentationType()["organizationRepresentative"].lstrip())
        return res if res else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())

class ContactTypeSF(RegistrantSortingField):
    _id="ContactType"

    def compare( self, r1, r2 ):
        # if same value, order by name
        res = cmp(r1.getRepresentationType()["contactType"], r2.getRepresentationType()["contactType"])
        return res if res else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())

class ParticipantRankSF(RegistrantSortingField):
    _id="ParticipantRank"

    def compare( self, r1, r2 ):
        # if same value, order by name
        # if empty, order at end
        rank1 = r1.getRank() if r1.getRank() else 999
        rank2 = r2.getRank() if r2.getRank() else 999
        res = cmp(rank1, rank2)
        return res if res else cmp( r1.getFamilyName().upper() + r1.getFirstName(), r2.getFamilyName().upper() + r2.getFirstName())


class SortingCriteria( filters.SortingCriteria ):
    """
    """
    _availableFields = {IdSF.getId():IdSF, \
                        NameSF.getId():NameSF, \
                        PositionSF.getId(): PositionSF, \
                        CitySF.getId():CitySF, \
                        PhoneSF.getId():PhoneSF, \
                        InstitutionSF.getId():InstitutionSF, \
                        EmailSF.getId():EmailSF, \
                        SessionsSF.getId():SessionsSF, \
                        AccommodationSF.getId():AccommodationSF, \
                        ArrivalDateSF.getId():ArrivalDateSF, \
                        DepartureDateSF.getId():DepartureDateSF, \
                        SocialEventsSF.getId():SocialEventsSF, \
                        ReasonParticipationSF.getId():ReasonParticipationSF, \
                        GeneralFieldSF.getId():GeneralFieldSF, \
                        StatusesSF.getId():StatusesSF, \
                        RegistrationDateSF.getId():RegistrationDateSF, \
                        CountrySF.getId():CountrySF, \
                        IsPayedSF.getId():IsPayedSF, \
                        IdPayment.getId():IdPayment, \
                        IsCheckedInSF.getId():IsCheckedInSF, \
                        CheckInDateSF.getId():CheckInDateSF,\
                        ProtocolSF.getId():ProtocolSF,\
                        RepresentationTypeSF.getId():RepresentationTypeSF,\
                        RepresentationSubTypeSF.getId():RepresentationSubTypeSF,\
                        OrganisationRepresentedSF.getId():OrganisationRepresentedSF,\
                        ParticipantRankSF.getId():ParticipantRankSF,\
                        ContactTypeSF.getId():ContactTypeSF}


    def __init__( self, crit = []):
        self._sortingField = None
        if crit:
            fieldKlass = self._availableFields.get( crit[0], None )
            if fieldKlass:
                self._sortingField = self._createField( fieldKlass )
            elif crit[0].startswith("s-"):
                self._sortingField = self._createField( StatusesSF )
                self._sortingField._specialId=crit[0]
            else:
                self._sortingField = self._createField( GeneralFieldSF )
                self._sortingField._specialId=crit[0]


