# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.

from cStringIO import StringIO
from tempfile import NamedTemporaryFile

from flask import session, request, flash

import MaKaC.webinterface.common.regFilters as regFilters
import MaKaC.webinterface.pages.registrants as registrants
import MaKaC.webinterface.pages.registrationForm as registrationForm
import MaKaC.webinterface.urlHandlers as urlHandlers
from MaKaC.webinterface.rh.fileAccess import RHFileAccess
from MaKaC.PDFinterface.conference import RegistrantsListToPDF,RegistrantsListToBookPDF, RegistrantsListToDOCX, RegistrantsGroupedListToDOCX
from MaKaC.export.excel import RegistrantsListToExcel
from MaKaC.errors import FormValuesError, UserError, ModificationError, AccessError
from MaKaC.common import utils
from MaKaC.registration import SocialEvent, MiscellaneousInfoGroup
from MaKaC.webinterface.common import registrantNotificator
from MaKaC.errors import MaKaCError, NotFoundError, NoReportError
from MaKaC.i18n import _
from MaKaC.webinterface.rh import registrationFormModif
from MaKaC.webinterface.rh.registrationFormModif import RHRegistrationFormModifBase
from indico.modules.payment.models.transactions import PaymentTransaction, TransactionAction
from indico.modules.payment.util import register_transaction
from indico.util.i18n import set_best_lang
from indico.util.printers import PrintingSystem
from indico.web.flask.util import send_file, url_for
from MaKaC.user import AvatarHolder,Avatar, LoginInfo
from MaKaC.registration import Registrant
from MaKaC.authentication import AuthenticatorMgr
from MaKaC.common import log
from MaKaC.common.cache import GenericCache
import csv
from indico.util import json

from MaKaC.webinterface.rh.templates import RHBadgeTemplates

class RHRegistrantListModifBase( registrationFormModif.RHRegistrationFormModifBase ):
    pass


class RHRegistrantListModif( RHRegistrantListModifBase ):
    """
    Registrant List - management area
    Handles filtering and display of different columns
    """

    _uh = urlHandlers.UHConfModifRegistrantList

    def _resetFilters( self, sessionData ):
        """
        Brings the filter data to a consistent state (websession),
        marking everything as "checked"
        """

        regForm = self._conf.getRegistrationForm()
        accommTypes = regForm.getAccommodationForm().getAccommodationTypesList()
        sessionData["accomm"] = map(lambda accom: accom.getId(),
                                    accommTypes)

        lstatuses = []
        for st in regForm.getStatusesList():
            lstatuses.append(st.getCaption() + st.getId() + "-NoValue")
            for stInt in st.getStatusValues():
                lstatuses.append(st.getCaption() + st.getId() + "-" +
                                 st.getStatusValues()[stInt].getCaption())
        sessionData["statuses"] = lstatuses

        sessTypes = regForm.getSessionsForm().getSessionList()
        sessionData["session"] = map(lambda session: session.getId(), sessTypes)

        socialEvents = regForm.getSocialEventForm().getSocialEventList()
        sessionData["event"] = map(lambda event: event.getId(), socialEvents)

        # By default, check '--none--'
        sessionData["accommShowNoValue"] = True
        sessionData["sessionShowNoValue"] = True
        sessionData["eventShowNoValue"] = True

        # Reset filter of representation section
        sessionData['representation'] = []

        # Reset filter of user sections
        for section in sessionData:
            if section.find("usersection")>-1:
                sessionData[section] = []

        # Reset statistics filter
        sessionData['contact'] = []
        sessionData['country'] = []
        if 'checked' in sessionData:
            del sessionData['checked']
        if 'printed' in sessionData:
            del sessionData['printed']

        return sessionData

    def _updateFilters( self, sessionData, params ):
        """
        Updates the filter parameters in the websession with those
        coming from the HTTP request
        """

        sessionData['event'] = []
        sessionData['accomm'] = []
        sessionData['statuses'] = []

        # Reset filter of representation section
        sessionData['representation'] = []

        # Reset filter of user sections
        for section in sessionData:
            if section.find("usersection")>-1:
                sessionData[section] = []

        # Reset statistics filter
        sessionData['contact'] = []
        sessionData['country'] = []
        if 'checked' in sessionData:
            del sessionData['checked']
        if 'printed' in sessionData:
            del sessionData['printed']

        sessionData.update(params)
        sessionData['session'] = utils.normalizeToList(params.get('session',[]))

        # update these elements in the session so that the parameters that are
        # passed are always taken into account (sessionData.update is not
        # enough, since the elements that are ommitted in params would just be
        # ignored

        sessionData['accommShowNoValue'] = params.has_key('accommShowNoValue')
        sessionData['eventShowNoValue'] = params.has_key('eventShowNoValue')
        sessionData['sessionShowNoValue'] = params.has_key('sessionShowNoValue')
        sessionData['firstChoice'] = params.has_key("firstChoice")

        return sessionData

    def _setStatFilters( self, sessionData, params ):
        """
        Set the filter parameters with the one coming from Stats view
        """
        if params.get('representationType', None):
                sessionData['representation'] = params.get('representationType')
        elif params.get('contactType', None):
                sessionData['contact'] = params.get('contactType')
        elif params.get('country', None):
                sessionData['country'] = params.get('country')
        elif params.get('checkedIn', None):
                sessionData['checked'] = params.get('checkedIn')
        elif params.get('printedBadge', None):
                sessionData['printed'] = params.get('printedBadge')
        elif params.get('statuses', None):
            criteria = params.get('statuses')
            if criteria=='Approved':
                sessionData['statuses'].remove('approvedapproved-No')
                sessionData['statuses'].remove('approvedapproved-NoValue')
            elif criteria=='Rejected':
                sessionData['statuses'].remove('rejectedrejected-No')
                sessionData['statuses'].remove('rejectedrejected-NoValue')
            elif criteria=='Panellist':
                sessionData['statuses'].remove('panellistpanel-No')
                sessionData['statuses'].remove('panellistpanel-NoValue')
            elif criteria=='Notprocessed':
                sessionData['statuses'].remove('approvedapproved-Yes')
                sessionData['statuses'].remove('approvedapproved-NoValue')
                sessionData['statuses'].remove('rejectedrejected-Yes')
                sessionData['statuses'].remove('rejectedrejected-NoValue')

        return sessionData

    def _buildFilteringCriteria(self, sessionData):
        """
        Creates the Filtering Criteria object, without changing the existing
        session data (sessionData is cloned, not directly changed)
        """
        sessionCopy = sessionData.copy()

        # filtering criteria needs a proper "filter name"
        sessionCopy.pop('session', None)
        sessionCopy[self._sessionFilterName] = sessionData.get('session')

        # add registrant filter
        sessionCopy['registrant'] = self._getRegistrarAffiliation()

        # Build the filtering criteria
        filterCrit = regFilters.RegFilterCrit(self._conf, sessionCopy)
        if sessionData.get('session'):
            filterCrit.getField("accomm").setShowNoValue(
                sessionCopy.get("accommShowNoValue") )
            filterCrit.getField(self._sessionFilterName).setShowNoValue(
                sessionCopy.get("sessionShowNoValue") )
            filterCrit.getField("event").setShowNoValue(
                sessionCopy.get("eventShowNoValue") )

        return filterCrit

    def _getRegistrarAffiliation(self):
        # get the limit affiliation of the registrar
        try:
            limit = self._conf.getRegistrarsLimits()[self.getAW().getUser().getId()]
            affiliation = limit['affiliation']
            return affiliation
        except (KeyError, AttributeError):
            return ''

    def _checkAction(self, params, filtersActive, sessionData, operation, isBookmark):
        """
        Decides what to do with the request parameters, depending
        on the type of operation that is requested
        """

        # user chose to reset the filters
        if operation ==  'resetFilters':
            self._filterUsed = False
            sessionData = self._resetFilters(sessionData)
            self._conf.setManagerCustomFilter(self.getAW().getUser().getId(), {})

        # set filters from Statistics view
        elif operation ==  'statFilter':
            sessionData = self._resetFilters(sessionData)
            self._filterUsed = True
            sessionData = self._setStatFilters(sessionData, params)

        # user set the filters
        elif operation ==  'setFilters':
            self._filterUsed = True
            sessionData = self._updateFilters(sessionData, params)
            filter = params.copy()
            del filter['operationType']
            del filter['OK']
            del filter['confId']
            self._conf.setManagerCustomFilter(self.getAW().getUser().getId(), filter)

        # user has changed the display options
        elif operation == 'setDisplay':
            self._filterUsed = filtersActive
            sessionData['disp'] = params.get('disp',[])
            self._conf.setManagerCustomDisplay(self.getAW().getUser().getId(), params.get('disp',[]))

        # session is empty (first time)
        # elif not filtersActive:
        elif not sessionData:
            self._filterUsed = False
            sessionData = self._resetFilters(sessionData)
            if self.getAW().getUser():
                # get user custom sorting
                customSorting = self._conf.getManagerCustomSorting(self.getAW().getUser().getId())
                sessionData['sortBy'] = customSorting[0] if customSorting else 'Name'
                sessionData['order'] = customSorting[1] if customSorting else 'down'
                # get user custom filter
                customFilter = self._conf.getManagerCustomFilter(self.getAW().getUser().getId())
                if customFilter:
                    sessionData.update(customFilter)
                    self._filterUsed = True
        else:
            self._filterUsed = filtersActive

        # if this is accessed through a direct link, the session is empty, so set default values
        if isBookmark:
            sessionData = self._resetFilters(sessionData)
            if operation !=  'resetFilters':
                sessionData = self._updateFilters(sessionData, params)
            sessionData['disp'] = params.get('disp',[])

        if ('order' in params or 'sortBy' in params) and not operation:
            sessionData['order'] = params.get('order', 'down')
            sessionData['sortBy'] = params.get('sortBy', 'Name')
            self._conf.setManagerCustomSorting(self.getAW().getUser().getId(), sessionData['sortBy'], sessionData['order'])

        # preserve the order and sortBy parameters, whatever happens
        if 'order' not in sessionData:
            sessionData['order'] = 'down'
        if 'sortBy' not in sessionData:
            sessionData['sortBy'] = 'Name'

        return sessionData

    def _checkParams(self, params):
        """
        Main parameter checking routine
        """

        RHRegistrantListModifBase._checkParams(self, params)

        operationType = params.get('operationType')

        # session data
        sessionData = session.get('registrantsFilterAndSortingConf%s' % self._conf.getId())

        # check if there is information already
        # set in the session variables
        if sessionData:
            # work on a copy
            sessionData = sessionData.copy()
            filtersActive = sessionData['filtersActive']
        else:
            # set a default, empty dict
            sessionData = {}
            filtersActive = False

        if 'resetFilters' in params:
            operation = 'resetFilters'
        elif operationType == 'statFilter':
            operation = 'statFilter'
        elif operationType == 'filter':
            operation = 'setFilters'
        elif operationType == 'display':
            operation = 'setDisplay'
        else:
            operation = None

        # the filter name will be different, depending
        # on whether only  the first choice for the session
        # is taken into account

        if 'firstChoice' in params:
            self._sessionFilterName = "sessionfirstpriority"
        else:
            self._sessionFilterName = "session"

        isBookmark = 'isBookmark' in params
        sessionData = self._checkAction(params, filtersActive, sessionData, operation, isBookmark)

        # Maintain the state abotu filter usage
        sessionData['filtersActive'] = self._filterUsed

        # Save the web session
        session['registrantsFilterAndSortingConf%s' % self._conf.getId()] = sessionData

        self._filterCrit = self._buildFilteringCriteria(sessionData)
        self._sortingCrit = regFilters.SortingCriteria([sessionData.get("sortBy", "Name").strip()])
        self._order = sessionData.get("order", "down")
        self._display = utils.normalizeToList(sessionData.get("disp", []))

    def _process(self):
        p = registrants.WPConfModifRegistrantList(self, self._conf, self._filterUsed)
        return p.display(filterCrit=self._filterCrit, sortingCrit=self._sortingCrit, display=self._display,
                         sessionFilterName=self._sessionFilterName, order=self._order)


class RHRegistrantListModifAction( RHRegistrantListModifBase ):
    _mapAvatarValues = {
        'Title': 'title',
        'Gender': 'gender',
        'First name': 'name',
        'Last name': 'surName',
        'Organization': 'organisation',
        'Birthdate (dd/mm/yyyy)': 'birthDate',
        'Communication email': 'email',
        'Address (professional)': 'address',
        'Communication phone': 'phone',
        'Communication fax': 'fax',
        'Passport number': 'passportId',
        'Nationality': 'passportOrigin',
        'Validity (dd/mm/yyyy)': 'passportExpire'
    }

    _mapRegistrantValues = {
        'Title': 'title',
        'Gender': 'gender',
        'First name': 'firstName',
        'Last name': 'surname',
        'Job title': 'position',
        'Organization': 'institution',
        'Birthdate (dd/mm/yyyy)': 'birthDate',
        'Address (professional)': 'address',
        'City (professional)': 'city',
        'Country (professional)': 'country',
        'Communication phone': 'phone',
        'Communication fax': 'fax',
        'Communication email': 'email',
        'Passport number': 'passportId',
        'Nationality': 'passportOrigin',
        'Validity (dd/mm/yyyy)': 'passportExpire',
        'Represented country': 'countryRepresentative',
        'Representation type': 'repType',
        'Representation subtype': 'repSubType',
        'Contact type': 'contactType',
        'Organization representative': 'organizationRepresentative'
    }

    def _checkParams( self, params ):
        RHRegistrantListModifBase._checkParams(self, params)
        set(params.keys())-{'confId','registrant','pdf.table','pdf.book'}
        cond1 = request.method == 'GET'
        cond2 = len(set(params.keys())-{'confId','registrant','pdf.table','pdf.book'}) > 0
        if cond1 and cond2:
            raise NoReportError('Cannot use GET in this case')
        self._windowsAgent = request.user_agent.platform == 'windows'
        self._selectedRegistrants = self._normaliseListParam(params.get("registrant",[]))
        self._addNew = params.has_key("newRegistrant")
        self._existingId = params.get("existingId") if self._addNew else None
        self._remove = params.has_key("removeRegistrants")
        self._email = params.has_key("email.x")
        self._emailSelected = params.has_key("emailSelected")
        self._tablePDF = params.has_key("pdf.table")
        self._allWord = params.has_key("word.all")
        self._provisionalWord = params.has_key("word.provisional")
        self._bookPDF = params.has_key("pdf.book")
        self._excel = params.has_key("excel")
        self._excel_reduced = params.has_key("excel_reduced")
        self._csv = params.has_key("csv")
        self._reglist = params.get("reglist","").split(",")
        self._display = self._normaliseListParam(params.get("disp",[]))
        self._printBadgesSelected = params.has_key("printBadgesSelected")
        self._package = params.has_key("PKG")
        self._approve = params.has_key("approveSelected")
        self._reject = params.has_key("rejectSelected")
        self._importCsv = params.has_key("importCsv")

    def _mapAvatar(self,data):
        result = {}
        for key in data:
            if self._mapAvatarValues.has_key(key) and data[key]!='':
                newKey = self._mapAvatarValues[key]
                result[newKey]=data[key]
        return result

    def _mapRegistrant(self,data):
        result = {}
        for key in data:
            if data[key]!='':
                if self._mapRegistrantValues.has_key(key):
                    newKey = self._mapRegistrantValues[key]
                    result[newKey]=data[key]
        return result

    def _setAffiliation(self, registrant):
        # set same affiliation for registrant as registrar affiliation
        try:
            limit = self._conf.getRegistrarsLimits()[self.getAW().getUser().getId()]
            registrant._representationType['organizationRepresentative'] = limit['affiliation']
        except KeyError:
            return

    def logimport(self,successfuls,unsuccessfuls):
        logInfo = {}
        logInfo["subject"] =  _("Completed import procedure: {} success, {} failed").format(len(successfuls),len(unsuccessfuls))
        logInfo["Success"] = successfuls
        logInfo["Unsuccess"] = unsuccessfuls
        self._conf.getLogHandler().logAction(logInfo, log.ModuleNames.REGISTRATION)

    def _processImportData(self, row):
        if row['Communication email']=='':
            raise UserError( _("Email is mandatory"))
        if row["Representation type"]=='':
            raise UserError( _("Representation Type is mandatory"))
        if row["Representation type"]=="Government" and row["Represented country"]=="":
            raise UserError( _("Represented Country is mandatory for Government representative"))

        row['Communication email']= row['Communication email'].lower()
        row['Password']= None
        if 'Nationality' not in row or row['Nationality']=='':
            row['Nationality']= row['Country (professional)']
        if "Representation subtype" in row and row["Representation subtype"][:14]=="IGO accredited":
                row["Representation subtype"]= "IGO accredited"
        if "Representation subtype" in row and row["Representation subtype"][:18]=="IGO non-accredited":
                row["Representation subtype"]= "IGO non-accredited"

        # set Organization Representative same as Institution
        if ("Organization representative" not in row or row["Organization representative"]=='') and \
            ("Organization" in row and row["Organization"]!=''):
            row["Organization representative"]= row["Organization"]

        # set Gender based on Title
        if 'Gender' not in row or row['Gender']=='':
            row['Gender'] = 'Female' if row['Title'] in ['Ms.','Mme.','Sra.'] else 'Male'

        # set default value for Contact Type
        if 'Contact type' not in row or row['Contact type']=='':
            from MaKaC.webinterface.listmapping import UNListMapping
            row['Contact type'] = UNListMapping.getContactTypeFromRep(row["Representation type"])

    def _lookupRegistrantInConf(self, row):
        for reg in self._conf.getRegistrantsList(): #lookup with Passport or Email
            if (row['Passport number']!='' and reg.getPassportId()==row['Passport number']) or (reg.getEmail()==row['Communication email']):
                return reg
        return None

    def _importXLSX(self,file):
        import datetime, xlrd
        from xlrd import open_workbook

        content = file.read()

        outfile = NamedTemporaryFile()
        outfile.write(content)
        outfile.seek(0)

        wb = open_workbook(outfile.name)
        sheet = wb.sheet_by_index(0)
        number_of_rows = sheet.nrows
        number_of_columns = sheet.ncols

        items = []
        names = []

        for col in range(number_of_columns):
            value = sheet.cell(0,col).value
            try:
                value = str(int(value))
            except ValueError:
                pass
            finally:
                names.append(value.encode('utf-8').strip())

        for row in range(1, number_of_rows):
            values = {}
            for col in range(number_of_columns):
                cell = sheet.cell(row,col)
                value = cell.value
                try:
                    if cell.ctype == xlrd.XL_CELL_DATE: # set date in format dd/mm/yyyy expected by setBirthDate()
                        date = datetime.datetime(1899, 12, 30)
                        get_ = datetime.timedelta(int(value))
                        get_col2 = str(date + get_)[:10]
                        d = datetime.datetime.strptime(get_col2, '%Y-%m-%d')
                        get_col = d.strftime('%d/%m/%Y')
                    else:
                        get_col = str(int(value))
                except:
                    get_col = value.encode('utf-8').strip()
                finally:
                    values[names[col]]=get_col

            items.append(values)
        outfile.close()
        return items

    def _import(self, file):
        # check Registration period
        import datetime
        import pytz
        utc= pytz.UTC
        startDate= self._conf.getRegistrationForm().getStartRegistrationDate()
        endDate = self._conf.getRegistrationForm().getEndRegistrationDate()
        current = datetime.datetime.now()
        current= utc.localize(current)

        if(current < startDate or current > endDate):
            raise NoReportError("Import registrants not authorized, outside registration period.")

        reader = self._importXLSX(file)
        #reader = csv.DictReader(file)
        i=1
        errors = []

        successfuls = []
        unsuccessfuls = []

        for row in reader:
            try:
                newReg = False
                user_id = None
                self._processImportData(row)
                matchedUsers = AvatarHolder().match({"email": row['Communication email']}, exact=1)
                if matchedUsers:
                    user = matchedUsers[0]
                elif(self._conf.getRegistrationForm().isMandatoryAccount()): #account mandatory for registration
                    avData = self._mapAvatar(row)
                    user = Avatar(avData)
                    AvatarHolder().add(user)
                    user.activateAccount()
                    login_info = LoginInfo(row['Communication email'], row['Password']) # login = email
                    auth_mgr = AuthenticatorMgr()
                    user_id = auth_mgr.createIdentity(login_info, user, "Local")
                    auth_mgr.add(user_id)
                else:
                    user = None

                reg = self._lookupRegistrantInConf(row)
                if not(reg):
                    if not(user): # not registered and account not mandatory
                        reg = Registrant()  #new registration
                        self._conf.addRegistrant(reg,user)
                        newReg = True
                    else: # not registered with an avatar
                        reg = Registrant()
                        reg.setAvatar(user)
                        self._conf.addRegistrant(reg,user)
                        user.addRegistrant(reg)
                        newReg = True
                        user.linkTo(reg, "registrant")

                regData = self._mapRegistrant(row)
                regData['import'] = 'import'
                reg.setValues(regData, user)
                self._setAffiliation(reg)
                successfuls.append(reg.getFullName())
            except Exception, e:
                name = row['Last name'] + ', ' + row['First name']
                print 'Error import (' + str(i) + '): ' + name + ', ' + str(e)
                errors.append('(' + str(i) + ')-'+name+ ': ' +str(e))
                unsuccessfuls.append(name)
                if(newReg):
                    self._conf.removeRegistrant(reg.getId())
                if(user_id):
                    auth_mgr.removeIdentity(user_id)
            finally:
                i+=1
        self.logimport(successfuls,unsuccessfuls)
        return errors

    def _process( self ):
        set_best_lang()  # prevents from having a _LazyString when generating a pdf without session.lang set
        if self._addNew:
            self._redirect(RHRegistrantNewForm._uh.getURL(self._conf,self._existingId))
        elif self._remove:
            if len(self._selectedRegistrants)>0:
                wp = registrants.WPRegistrantModifRemoveConfirmation(self, self._conf, self._selectedRegistrants)
                return wp.display()
            else:
                self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        elif self._email:
            r = RHRegistrantListEmail(self, self._conf,self._reglist)
            return r.email()
        elif self._emailSelected:
            if len(self._selectedRegistrants)>0:
                r = RHRegistrantListEmail(self, self._conf,self._selectedRegistrants)
                return r.email()
            else:
                self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        elif self._tablePDF:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantListPDF(self,self._conf,regs, self._display)
            return r.pdf()
        elif self._allWord:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantAllWord(self,self._conf,regs, self._display)
            return r.docx()
        elif self._provisionalWord:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantProvisionalWord(self,self._conf,regs, self._display)
            return r.docx()
        elif self._bookPDF:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantBookPDF(self,self._conf,regs, self._display)
            return r.pdf()
        elif self._excel:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantListExcel(self,self._conf,regs, self._display, self._windowsAgent,csv=False)
            return r.excel()
        elif self._excel_reduced:
            regs = []
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantListExcel(self, self._conf, regs, self._display, self._windowsAgent, csv=False, reduced = True)
            return r.excel()

        elif self._csv:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantListExcel(self,self._conf,regs, self._display, self._windowsAgent,csv=True)
            return r.excel()
        elif self._printBadgesSelected:
            if len(self._selectedRegistrants) > 0:
                wp = registrants.WPRegistrantModifPrintBadges(self, self._conf, self._selectedRegistrants)
                return wp.display()
            else:
                self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        elif self._package:
            regs =[]
            for reg in self._selectedRegistrants:
                if self._conf.getRegistrantById(reg) != None:
                    regs.append(self._conf.getRegistrantById(reg))
            r = RHRegistrantPackage(self, self._conf,regs)
            return r.pack()
        elif self._approve:
            if len(self._selectedRegistrants)>0:
                wp = registrants.WPRegistrantModifApproveConfirmation(self, self._conf, self._selectedRegistrants)
                return wp.display()
            else:
                self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        elif self._reject:
            if len(self._selectedRegistrants)>0:
                wp = registrants.WPRegistrantModifRejectConfirmation(self, self._conf, self._selectedRegistrants)
                return wp.display()
            else:
                self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        elif self._importCsv:
            file = request.files['file']
            errors = self._import(file)
            if len(errors)>0:
                wp = registrants.WPRegistrantModifImportError(self, self._conf, errors)
                return wp.display()
            else:
                flash(_("All registrants have been successfully imported"),'success')

            self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))
        else:
            self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))

class RHRegistrantNewForm(RHRegistrantListModifBase):
    _uh = urlHandlers.UHConfModifRegistrantNew

    def _checkParams(self, params):
        RHRegistrantListModifBase._checkParams(self, params)
        # self._existingId = params.get("existingId")

    def _process(self):
        p = registrationForm.WPRegistrationFormDisplay(self, self._conf)
        return p.display()

class RHRegistrantListRemove(RHRegistrantListModifBase):

    def _checkParams( self, params ):
        RHRegistrantListModifBase._checkParams(self, params)
        self._selectedRegistrants = self._normaliseListParam(params.get("registrants",[]))
        self._cancel=params.has_key("cancel")

    def _process(self):
        if not self._cancel:
            for reg in self._selectedRegistrants:
                self._conf.removeRegistrant(reg)
                PaymentTransaction.query.filter_by(event_id=self._conf.getId(), registrant_id=reg).delete()
        self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))


class RHRegistrantListBulkModify(RHRegistrantListModifBase):

    def _checkParams( self, params ):
        RHRegistrantListModifBase._checkParams(self, params)
        self._params = { k: params[k] for k in params if params[k]}
        self._selectedRegistrants = params.get("registrants","").split(',')
        del self._params['confId']
        del self._params['registrants']

    def _process(self):
        for regId in self._selectedRegistrants:
            reg = self._conf.getRegistrantById(regId)
            oldReprType = reg.getRepresentationType()
            newReprType = dict(oldReprType.items()+self._params.items())
            reg.setRepresentationType(newReprType)
        self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))


class RHRegistrantListApprove(RHRegistrantListModifBase):

    def _checkParams( self, params ):
        RHRegistrantListModifBase._checkParams(self, params)
        self._selectedRegistrants = self._normaliseListParam(params.get("registrants",[]))
        self._cancel=params.has_key("cancel")
        self._errors = []

    def _process(self):
        if not self._cancel:
            for reg in self._selectedRegistrants:
                # check if Rejected
                st = self._conf.getRegistrantById(reg).getStatusByCaption('rejected')
                if(st.getStatusValue().getId()=='0'):
                    # regName = self._conf.getRegistrantById(reg).getSurName() + ', ' + self._conf.getRegistrantById(reg).getFirstName()
                    # self._errors.append('('+ regName + '): rejected user can not be approved')
                    # unset Rejected
                    st2 = self._conf.getRegistrantById(reg).getRegistrationForm().getStatusById(st.getId())
                    st.setStatusValue(st2.getStatusValueById('1'))

                # set Approved
                st = self._conf.getRegistrantById(reg).getStatusByCaption('approved')
                st2 = self._conf.getRegistrantById(reg).getRegistrationForm().getStatusById(st.getId())
                st.setStatusValue(st2.getStatusValueById('0'))

            # if len(self._errors)>0 :
            #     str_errors = ', '.join(self._errors)
            #     flash(_("Approve errors : {0}").format(str_errors),'error')

        self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))

class RHRegistrantListReject(RHRegistrantListModifBase):

    def _checkParams( self, params ):
        RHRegistrantListModifBase._checkParams(self, params)
        self._selectedRegistrants = self._normaliseListParam(params.get("registrants",[]))
        self._cancel=params.has_key("cancel")

    def _process(self):
        if not self._cancel:
            for reg in self._selectedRegistrants:
                # set Rejected
                st = self._conf.getRegistrantById(reg).getStatusByCaption('rejected')
                st2 = self._conf.getRegistrantById(reg).getRegistrationForm().getStatusById(st.getId())
                st.setStatusValue(st2.getStatusValueById('0'))
                # unset Approved
                st = self._conf.getRegistrantById(reg).getStatusByCaption('approved')
                st2 = self._conf.getRegistrantById(reg).getRegistrationForm().getStatusById(st.getId())
                st.setStatusValue(st2.getStatusValueById('1'))
        self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))

class RHRegistrantBookPDF:
    def __init__( self, rh,conf,reglist, disp ):
        self._conf = conf
        self._list = reglist
        self._rh = rh
        self._display = disp

    def pdf(self):
        pdf = RegistrantsListToBookPDF(self._conf, list=self._list, display=self._display)
        return send_file('RegistrantsBook.pdf', StringIO(pdf.getPDFBin()), 'PDF')


class RHRegistrantListPDF:
    def __init__( self, rh,conf,reglist, disp ):
        self._conf = conf
        self._list = reglist
        self._rh = rh
        self._display = disp

    def pdf(self):
        pdf = RegistrantsListToPDF(self._conf, list=self._list, display=self._display)
        try:
            data = pdf.getPDFBin()
        except:
            raise FormValuesError( _("""Text too large to generate a PDF with "Table Style". Please try again generating with "Book Style"."""))
        return send_file('RegistrantsList.pdf', StringIO(data), 'PDF')


class RHRegistrantAllWord:
    def __init__( self, rh,conf,reglist, disp ):
        self._conf = conf
        self._list = reglist
        self._rh = rh
        self._display = disp

    def docx(self):
        docx = RegistrantsListToDOCX(self._conf, list=self._list, display=self._display)
        try:
            docx.generate()
            data = docx.getDocx()
        except Exception as e:
            raise FormValuesError( _(e.message))
            # raise FormValuesError( _("""Text too large to generate a PDF with "Table Style". Please try again generating with "Book Style"."""))
        return send_file('RegistrantsList.docx', StringIO(data), 'application/vnd.openxmlformats-officedocument.wordprocessingml.document')


class RHRegistrantProvisionalWord:
    def __init__( self, rh,conf,reglist, disp ):
        self._conf = conf
        self._list = reglist
        self._rh = rh
        self._display = disp

    def docx(self):
        docx = RegistrantsGroupedListToDOCX(self._conf, list=self._list, display=self._display)
        try:
            docx.generate()
            data = docx.getDocx()
        except Exception as e:
            raise FormValuesError( _("""Text too large to generate a PDF with "Table Style". Please try again generating with "Book Style"."""))
        return send_file('RegistrantsList.docx', StringIO(data), 'application/vnd.openxmlformats-officedocument.wordprocessingml.document')


class RHRegistrantListExcel:
    def __init__( self, rh,conf,reglist, disp, excelSpecific,csv, reduced=False):
        self._conf = conf
        self._list = reglist
        self._rh = rh
        self._display = disp
        self._excelSpecific = excelSpecific
        self._csv = csv
        self._reduced = reduced

    def excel(self):
        excel = RegistrantsListToExcel(self._conf, list=self._list, display=self._display,
                                       excelSpecific=self._excelSpecific,csv=self._csv, reduced = self._reduced)
        name = 'RegistrantsList.'+('csv' if self._csv else 'xlsx')
        mime = 'CSV' if self._csv else 'XLSX'
        return send_file(name, StringIO(excel.getExcelFile()), mime)


class RHRegistrantListEmail:
    def __init__(self, rh, conf, reglist):
        self._conf = conf
        self._regList = reglist
        self._rh = rh

    def _checkParams(self, params):
        self._from = params.get('from', '')
        self._cc = params.get('cc', '')
        self._subject = params.get('subject', '')
        self._body = params.get('body', '')

    def email(self):
        self._checkParams(self._rh.getRequestParams())
        p=registrants.WPEMail(self._rh, self._conf, self._regList, self._from, self._cc, self._subject, self._body)
        return p.display()

class RHRegistrantModifBase( RHRegistrationFormModifBase ):

    def _checkProtection( self ):
        RHRegistrationFormModifBase._checkProtection(self)
        if not self._conf.hasEnabledSection("regForm"):
            raise MaKaCError( _("The registrants' management was disabled by the conference managers"))

    def _checkParams( self, params ):
        RHRegistrationFormModifBase._checkParams(self, params)
        regId=params.get("registrantId",None)
        if regId is None:
            raise MaKaCError(_("registrant id not set"))
        self._registrant=self._conf.getRegistrantById(regId)
        if self._registrant is None:
            raise NotFoundError(_("The registrant with id %s does not exist or has been deleted")%regId)

    def logUpdate(self,modType):
        logInfo = {}
        logInfo["subject"] =  _("Modified {} for registrant: {}").format(modType,self._registrant.getFullName())
        self._conf.getLogHandler().logAction(logInfo, log.ModuleNames.REGISTRATION)

class RHRegistrantModification( RHRegistrantModifBase ):

    def _process( self ):
        p = registrants.WPRegistrantModification( self, self._registrant )
        return p.display()


class RHRegistrantModificationEticket(RHRegistrantModifBase):

    def _process(self):
        p = registrants.WPRegistrantModifETicket(self, self._registrant)
        return p.display()


class RHRegistrantModificationEticketCheckIn(RHRegistrantModifBase):

    def _checkParams(self, params):
        RHRegistrantModifBase._checkParams(self, params)
        self._newStatus = params["changeTo"]

    def _process(self):
        eticket = self._conf.getRegistrationForm().getETicket()
        if self._newStatus == "True":
            self._registrant.setCheckedIn(True)
        else:
            self._registrant.setCheckedIn(False)
        self._redirect(url_for("event_mgmt.confModifRegistrants-modification-eticket", self._registrant))


class RHRegistrantSendEmail( RHRegistrationFormModifBase ):

    def _checkParams(self, params):
        RHRegistrationFormModifBase._checkParams( self, params )
        self._regsIds = self._normaliseListParam(params.get("regsIds",[]))
        self._cancel=params.has_key("cancel")
        self._send = params.has_key("OK")
        self._preview = params.has_key("preview")
        if not self._cancel:
            if params.get("from","") == "":
                raise FormValuesError("Please write from address")
            if params.get("subject","") == "":
                raise FormValuesError("Please write any subject for the email")
            if params.get("body","") == "":
                raise FormValuesError("Please write any body for the email")
            self._p={}
            self._p["subject"]=params["subject"]
            self._p["from"]=params["from"]
            self._p["body"]=params["body"]
            self._p["cc"]=[params.get("cc", "")]

    def _process(self):
        if self._send:
            for regId in self._regsIds:
                reg=self._conf.getRegistrantById(regId)
                if  reg!=None:
                    self._p['to']=[reg.getEmail()]
                    self._p['conf'] = self._conf
                    registrantNotificator.EmailNotificator().notify(reg, self._p)
            p = registrants.WPSentEmail(self, self._target)
            return p.display()
        elif self._preview:
            p = registrants.WPPreviewEmail(self, self._target, self._getRequestParams())
            return p.display()
        else:
            self._redirect(urlHandlers.UHConfModifRegistrantList.getURL(self._conf))

class RHRegistrantPackage:

    def __init__( self, rh, conf, reglist ):
        self._conf = conf
        self._list = reglist
        self._rh = rh

    def pack(self):
        from MaKaC.common.contribPacker import ZIPFileHandler, RegistrantPacker
        p = RegistrantPacker(self._conf)
        path = p.pack(self._list, ZIPFileHandler())
        return send_file('registrants.zip', path, 'ZIP', inline=False)


class RHRegistrantPerformDataModification( RHRegistrantModifBase ):

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._regForm = self._conf.getRegistrationForm()
        pd = self._regForm.getPersonalData()
        keys = pd.getMandatoryItems()
        for key in keys:
            if key not in params.keys() or params.get(key,"").strip() == "":
                raise FormValuesError(_("The field \"%s\" is mandatory and you must fill it in order to modify the registrant")%(pd.getData()[key].getName()))
        self._cancel = params.has_key("cancel")

    def _process( self ):
        if not self._cancel:
            if self._getRequestParams().get("email","") != self._registrant.getEmail() and self._conf.hasRegistrantByEmail(self._getRequestParams().get("email","")):
                raise FormValuesError(_("There is already a user with the email \"%s\". Please choose another one")%self._getRequestParams().get("email","--no email--"))
            self._registrant.setPersonalData(self._getRequestParams())
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))


class RHRegistrantSessionModify( RHRegistrantModifBase ):

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")

    def _process( self ):
        if not self._conf.getRegistrationForm().getSessionsForm().isEnabled():
            self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        else:
            p = registrants.WPRegistrantSessionModify(self, self._registrant)
            return p.display()


class RHRegistrantTransactionPerformModify(RHRegistrantModifBase):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams(self, params):
        RHRegistrantModifBase._checkParams(self, params)
        self._isPayed = params["isPayed"]

    def _process(self):
        action = TransactionAction.complete if self._isPayed == '1' else TransactionAction.cancel
        register_transaction(registrant=self._registrant,
                             amount=self._registrant.getTotal(),
                             currency=self._registrant.getCurrency(),
                             action=action,
                             provider='_manual',
                             data={'changed_by_name': session.user.getFullName(),
                                   'changed_by_id': session.user.getId()})
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))


class RHRegistrantSessionPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        if not self._cancel:
            self._regForm = self._conf.getRegistrationForm()
            sessionForm = self._regForm.getSessionsForm()
            if not sessionForm.isEnabled():
                self._cancel = True
                return
            self._sessions = sessionForm.getSessionsFromParams(params)

    def _process( self ):
        if not self._cancel:
            if not self._registrant.getPayed():
                sessions = self._sessions
            else:
                # First keep all sessions which are billable (they are not submitted anymore)
                sessions = [session for session in self._registrant.getSessionList() if session.isBillable()]
                # Then take all chosen sessions which are not billable
                sessions += [session for session in self._sessions if not session.isBillable()]
            self._registrant.setSessions(sessions)
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantAccommodationModify( RHRegistrantModifBase ):

    def _process( self ):
        if not self._conf.getRegistrationForm().getAccommodationForm().isEnabled():
            self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        else:
            p = registrants.WPRegistrantAccommodationModify(self, self._registrant)
            return p.display()

class RHRegistrantAccommodationPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        if not self._cancel:
            self._regForm = self._conf.getRegistrationForm()
            if not self._regForm.getAccommodationForm().isEnabled():
                self._cancel = True
                return
            self._arrivalDate = params.get("arrivalDate",None)
            self._departureDate = params.get("departureDate",None)
            if self._arrivalDate is not None and self._departureDate is not None:
                self._arrivalDate = utils.stringToDate(self._arrivalDate)
                self._departureDate = utils.stringToDate(self._departureDate)
                if self._arrivalDate > self._departureDate:
                    raise FormValuesError("Arrival date has to be earlier than departure date")
            self._accoType = self._regForm.getAccommodationForm().getAccommodationTypeById(params.get("accommodation_type", ""))
            if self._regForm.getAccommodationForm().getAccommodationTypesList() !=[] and self._accoType is None:
                raise FormValuesError("It is mandatory to choose an accommodation in order to register")

    def _process( self ):
        if not self._cancel:
            # Allow changing of the dates only if the current accomodation is not billable or the user hasn't paid yet
            currentAccoType = self._registrant.getAccommodation().getAccommodationType()
            if not self._registrant.getPayed() or currentAccoType is None or not currentAccoType.isBillable():
                self._registrant.getAccommodation().setArrivalDate(self._arrivalDate)
                self._registrant.getAccommodation().setDepartureDate(self._departureDate)
            if self._regForm.getAccommodationForm().getAccommodationTypesList() !=[]:
                # Only change the accommodation type if:
                # - the registrant hasn't paid yet OR
                # - neither the current nor the new accommodation is billable
                if not self._registrant.getPayed() or \
                    ((currentAccoType is None or not currentAccoType.isBillable()) and \
                     (self._accoType is None or not self._accoType.isBillable())):
                    self._registrant.getAccommodation().setAccommodationType(self._accoType)
        self._registrant.updateTotal()
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantSocialEventsModify( RHRegistrantModifBase ):

    def _process( self ):
        if not self._conf.getRegistrationForm().getSocialEventForm().isEnabled():
            self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        else:
            p = registrants.WPRegistrantSocialEventsModify(self, self._registrant)
            return p.display()

class RHRegistrantSocialEventsPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        if not self._cancel:
            self._regForm = self._conf.getRegistrationForm()
            if not self._regForm.getSocialEventForm().isEnabled():
                self._cancel = True
            else:
                socialEventIds = self._normaliseListParam(params.get("socialEvents", []))
                self._socialEvents = []
                self._places = {}
                for id in socialEventIds:
                    self._socialEvents.append(self._regForm.getSocialEventForm().getSocialEventById(id))
                    self._places[id] = params.get("places-%s"%id, "1")

    def _process( self ):
        if not self._cancel:
            for seItem in self._registrant.getSocialEvents()[:]:
                # Remove all items which can be added back (i.e. if paid only non-billable ones)
                if not (self._registrant.getPayed() and seItem.isBillable()):
                    self._registrant.removeSocialEventById(seItem.getId())

            l = []
            for seItem in self._socialEvents:
                # Only add item if the registrant hasn't paid yet or the item is not billable
                if not self._registrant.getPayed() or not seItem.isBillable():
                    newSE = SocialEvent(seItem, int(self._places[seItem.getId()]))
                    self._registrant.addSocialEvent(newSE)
        self._registrant.updateTotal()
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantReasonParticipationModify( RHRegistrantModifBase ):

    def _process( self ):
        if not self._conf.getRegistrationForm().getReasonParticipationForm().isEnabled():
            self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        else:
            p = registrants.WPRegistrantReasonParticipationModify(self, self._registrant)
            return p.display()

class RHRegistrantReasonParticipationPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        if not self._cancel:
            self._regForm = self._conf.getRegistrationForm()
            if not self._regForm.getReasonParticipationForm().isEnabled():
                self._cancel = True
            else:
                self._reason = params.get("reason","")

    def _process( self ):
        if not self._cancel:
            self._registrant.setReasonParticipation(self._reason)
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantRepTypeModify( RHRegistrantModifBase ):

    def _process( self ):
        if not self._conf.getRegistrationForm().getRepresentationForm().isEnabled():
            self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        else:
            p = registrants.WPRegistrantRepTypeModify(self, self._registrant)
            p.addExtraJS(self._JSRepTypeModify())   # if possible, do include script instead of adding code
            return p.display()

    def _JSRepTypeModify(self):
        from indico.util.i18n import i18nformat
        script = i18nformat("""
                """)
        return script

class RHRegistrantRepTypePerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        if not self._cancel:
            self._regForm = self._conf.getRegistrationForm()
            if not self._regForm.getRepresentationForm().isEnabled():
                self._cancel = True
                return
            self._countryRepresentative = params.get("countryRepresentative",None)
            self._repType = params.get("repType",None)
            self._repSubType = params.get("repSubType",None)
            self._contactType = params.get("contactType",None)
            self._organizationRepresentative = params.get("organizationRepresentative",None)
            if not(self._contactType) or (self._contactType=='?'):
                raise FormValuesError("It is mandatory to select a Contact Type in order to register")
            if not(self._repType) or (self._repType=='?'):
                raise FormValuesError("It is mandatory to select a Representation Type in order to register")
            if (self._repType=="Government") and not(self._countryRepresentative):
                raise FormValuesError("It is mandatory to select a country for a government representative")

    def _process( self ):
        if not self._cancel:
            data = {}
            data['countryRepresentative'] = self._countryRepresentative
            data['repType'] = self._repType
            data['repSubType'] = self._repSubType
            data['contactType'] = self._contactType
            data['organizationRepresentative'] = self._organizationRepresentative
            self._registrant.setRepresentationType(data)
            self.logUpdate("representation type")
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantMiscInfoModify( RHRegistrantModifBase ):

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._miscInfoId = params.get("miscInfoId","")
        self._miscInfo=self._registrant.getMiscellaneousGroupById(self._miscInfoId)

    def _process( self ):
        gsf=self._conf.getRegistrationForm().getGeneralSectionFormById(self._miscInfoId)
        if gsf is not None:
            if self._miscInfo is None:
                self._miscInfo=MiscellaneousInfoGroup(self._registrant, gsf)
                self._registrant.addMiscellaneousGroup(self._miscInfo)
            p = registrants.WPRegistrantMiscInfoModify(self, self._miscInfo)
            return p.display()
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))

class RHRegistrantMiscInfoPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        miscInfoId = params.get("miscInfoId","")
        self._miscInfo=self._registrant.getMiscellaneousGroupById(miscInfoId)

    def _process( self ):
        if not self._cancel:
            params = self._getRequestParams()
            pdForm = self._registrant.getRegistrationForm().getPersonalData()
            if self._miscInfo.getGeneralSection() is pdForm:
                email = pdForm.getValueFromParams(params, 'email')
                if email != self._registrant.getEmail() and self._conf.hasRegistrantByEmail(email):
                    raise FormValuesError("""There is already a user with the email "%s". Please choose another one""" % email)
            for f in self._miscInfo.getGeneralSection().getSortedFields():
                if not f.isDisabled():
                    f.getInput().setResponseValue(self._miscInfo.getResponseItemById(f.getId()), params, self._registrant, self._miscInfo)
            if self._miscInfo.getGeneralSection() is pdForm:
                self._registrant.setPersonalData(pdForm.getRegistrantValues(self._registrant))
        self._registrant.updateTotal()
        self.logUpdate("personal data")
        self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))


class RHRegistrantStatusesModify( RHRegistrantModifBase ):

    def _process( self ):
        p = registrants.WPRegistrantStatusesModify(self, self._registrant)
        return p.display()

class RHRegistrantStatusesPerformModify( RHRegistrantModifBase ):
    _uh = urlHandlers.UHRegistrantModification

    def _checkProtection( self ):
        user = self.getAW().getUser()

        if self._target.isRegistrar(user):
            rights = self._target.getRegistrarRights(user.getId())
            if "Approve" in rights['Right']:
                return
            else:
                raise AccessError( _("Approval right not present"))

        RHRegistrantModifBase._checkProtection(self)

    def _checkParams( self, params ):
        RHRegistrantModifBase._checkParams(self, params)
        self._cancel = params.has_key("cancel")
        self._statuses={}
        for pkey in params.keys():
            if pkey.startswith("statuses-"):
                id=pkey.split("-")[1]
                self._statuses[id]=params.get(pkey)

    def notifyOnApproval(self,statuses):

        notify = False
        statusToNotify=""
        if "approved" in statuses and statuses["approved"]=="0":
            notify = True
            statusToNotify="approved"
            templateName = "Approval"

        if "rejected" in statuses and statuses["rejected"]=="0":
            notify = True
            statusToNotify="rejected"
            templateName = "Reject"

        if notify:
            reg = self._registrant
            if reg:
                if self._conf.getRegistrationForm().isNotificationApproval():
                    from MaKaC.webinterface.mail import generic_send
                    generic_send(reg, templateName, self._conf,
                             sender=self._conf.getRegistrationForm().getNotificationSender())
                    if 'email sent' not in reg.getTags():
                        l = reg.getTags()
                        l.append('email sent')
                        reg.setTags(l)
                        reg.notifyModification()

                logData = dict()
                logData["subject"] = _("{} has been {}").format(reg.getFullName(),statusToNotify)
                url = url_for('event_mgmt.confModifRegistrants-modification', reg, _external=True, _secure=True)
                logData["link"] = '<a href="{0}">{0}</a>'.format(url)
                self._conf.getLogHandler().logAction(logData,log.ModuleNames.REGISTRATION)


    def _process( self ):
        try:
            if not self._cancel:
                for stkey in self._statuses.keys():
                    st=self._conf.getRegistrationForm().getStatusById(stkey)
                    if st is not None:
                        v=st.getStatusValueById(self._statuses[stkey])
                        rst=self._registrant.getStatusById(stkey)
                        rst.setStatusValue(v)
                self.notifyOnApproval(self._statuses)
                self._redirect(urlHandlers.UHRegistrantModification.getURL(self._registrant))
        except Exception as e:
            self._responseUtil.status = 404


class RHGetAttachedFile(RHFileAccess, RHRegistrationFormModifBase):

    def _checkProtection( self ):
        if 'UNOG Registrant Kiosk' not in request.headers.get('User-Agent'):
            if self._target.getOwner().getAvatar() != self._getUser() or self._getUser() is None:
                tempTarget = self._target
                self._target = self._conf
                RHRegistrationFormModifBase._checkProtection(self)
                self._target = tempTarget

# new class for mobile print badge service
class RHRegistrantMobilePrintBadge(RHBadgeTemplates):
    # need to define attributes because not inherited from parent class
    _marginTop = None
    _marginBottom = None
    _marginLeft = None
    _marginRight = None
    _marginColumns = None
    _marginRows = None
    _pagesize = None
    _drawDashedRectangles = None
    _landscape = None
    _template = None

    def _checkParams(self, params):
        import MaKaC.conference as conference

        self._conf = conference.ConferenceHolder().getById(params.get("confId", None))
        self._registrantId = params.get("registrantId", None)
        self._registrant = self._conf.getRegistrantById(self._registrantId)
        if(self._registrant):
            defaultConf = conference.CategoryManager().getDefaultConference()
            self._setBadgeParams(self._registrant, defaultConf)
        else:
            self._registrantId = None

    def _checkProtection(self):
        from MaKaC.errors import AccessError
        if not session.user and not self._doProcess:
            return
        if not(session.user) or (not session.user.is_member_of_group("security_officers") and not session.user.isAdmin()):
            raise AccessError("PrintBadge")

    def _process(self):
        from MaKaC.PDFinterface.conference import RegistrantsListToBadgesPDF

        if (self._conf.isClosed()):
            return _("The conference is closed.")
        else:
            if self._registrantId == None:
                return  _("There is no such selected registrant.")
            elif self._templateId == None:
                return  _("There is no badge template for the selected registrant.")

            pdf = RegistrantsListToBadgesPDF(self._conf,
                                             self._template,
                                             self._marginTop,
                                             self._marginBottom,
                                             self._marginLeft,
                                             self._marginRight,
                                             self._marginColumns,
                                             self._marginRows,
                                             self._pagesize,
                                             self._drawDashedRectangles,
                                             [self._registrantId],
                                             self._landscape)

            result = self._printBadge(pdf)
            self.logPrintedBadge(result)
            return result

    def getLogInfo(self):
        logInfo = dict()

        logInfo["subject"] =  "Printed badge for registrant with ID: {} ({})".format(self._registrantId,self._registrant.getFullName())
        logInfo["full name"] = self._registrant.getFullName()
        canEnter = self._registrant.canEnterPremises()
        logInfo["status"] = canEnter[0]
        logInfo["reason"] = canEnter[1]

        return logInfo

    def logPrintedBadge(self,subsystemResult=None):
        logInfo = self.getLogInfo()
        logInfo["printing result"]=subsystemResult if isinstance(subsystemResult, basestring) else "Ok"
        self._registrant.increasePrintedBagde()
        self._conf.getLogHandler().logAction(logInfo, log.ModuleNames.REGISTRATION)

    def _printBadge(self, pdf):
        import base64
        # this method will be overwritten when called from <Print Badge> to print directly on screen
        #return _("Badge printed.") if(pdf.printBadge(pdf.getPDFBin())==0) else _("Error print badge.")
        return send_file('Badges.pdf', StringIO(base64.b64encode(pdf.getPDFBin())) , 'PDF')


    def _setBadgeParams(self, registration, defaultConf):
        # settings for the template to print

        pdfOptions = defaultConf.getBadgeTemplateManager().getPDFOptions()
        self._marginTop = pdfOptions.getTopMargin()
        self._marginBottom = pdfOptions.getBottomMargin()
        self._marginLeft = pdfOptions.getLeftMargin()
        self._marginRight = pdfOptions.getRightMargin()
        self._marginColumns = pdfOptions.getMarginColumns()
        self._marginRows = pdfOptions.getMarginRows()
        self._pagesize = "A4"
        self._drawDashedRectangles = False
        self._landscape = False

        self._templateId,self._template = registration.getBadgeTemplate()


class RHRegistrantPrintBadge(RHRegistrantMobilePrintBadge):

    def _checkParams(self, params):
        RHRegistrantMobilePrintBadge._checkParams(self,params)
        self._base64 = params.get("base64","")
        self._printer = params.get("printer",None)
        self._isMobile = params.get("ismobile","") == "true"

    def _printBadge(self, pdf):
        printOnServer = PrintingSystem.getInstance().printOnServer()

        if printOnServer or self.isMobile():
            printer = self._printer if self._printer else PrintingSystem.getInstance().printer()

            (status,msgOrJob) = pdf.printBadge(pdf.getPDFBin(),printer)
            msg = _("Badge printed.") if status is True else _(msgOrJob)
            if not self.isMobile():
                flash(msg)
                url = urlHandlers.UHRegistrantModification.getURL(self._conf)
                url.addParam("registrantId", str(self._registrantId))
                return self._redirect(url)
            else:
                return msg
        elif self.isMobile() is False:
            import base64
            payload = pdf.getPDFBin()
            content = base64.b64encode(payload) if self._base64 else payload
            return send_file('Badges.pdf', StringIO(content), 'application/pdf')

    def getLogInfo(self):
        logInfo = RHRegistrantMobilePrintBadge.getLogInfo(self)
        logInfo["mobile"]=self._isMobile
        return logInfo
