# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.
from cStringIO import StringIO

from flask import session, request, flash, redirect

from MaKaC.webinterface.rh.conferenceDisplay import RHConferenceBaseDisplay, RHConfSignIn
import MaKaC.webinterface.urlHandlers as urlHandlers
import MaKaC.webinterface.pages.registrationForm as registrationForm
from MaKaC import registration
from MaKaC.errors import FormValuesError, MaKaCError, AccessError, NotFoundError
from indico.core.config import Config
from MaKaC.user import AvatarHolder
from MaKaC.webinterface.rh.registrantsModif import RHRegistrantListModif
from MaKaC.common.utils import validMail
from MaKaC.PDFinterface.conference import TicketToPDF
from indico.modules.events.registration.notifications import (notify_registration_confirmation,
                                                              notify_registration_modification)
from indico.modules.payment import event_settings as payment_event_settings
from indico.web.flask.util import send_file, url_for
from indico.util import json
from indico.util.i18n import set_best_lang, _
from MaKaC.common import log
from MaKaC.webinterface.listmapping import UNListMapping


class RHBaseRegistrationForm(RHConferenceBaseDisplay):

    def _checkParams(self, params):
        RHConferenceBaseDisplay._checkParams(self, params)
        self._regForm = self._conf.getRegistrationForm()

    def _processIfActive(self):
        """only override this method if the RegForm must be activated for
            carrying on the handler execution"""
        return "regForm"

    def _process(self):
        #if the RegForm is not activated we show up a form informing about that.
        #   This must be done at RH level because there can be some RH not
        #   displaying pages.
        if not self._regForm.isActivated() or not self._conf.hasEnabledSection("regForm"):
            p = registrationForm.WPRegFormInactive(self, self._conf)
            return p.display()
        else:
            return self._processIfActive()


class RHRegistrationFormSignIn(RHBaseRegistrationForm, RHConfSignIn):
    _uh = urlHandlers.UHConfRegistrationFormSignIn

    def _checkParams(self, params):
        RHBaseRegistrationForm._checkParams(self, params)
        RHConfSignIn._checkParams(self, params)
        self._signInPage = registrationForm.WPRegistrationFormSignIn(self, self._conf)

    def _processIfActive(self):
        return self._makeLoginProcess()


class RHRegistrationFormDisplayBase(RHBaseRegistrationForm):
    _uh = urlHandlers.UHConfRegistrationFormDisplay

    def _getLoginURL(self):
        urlLogin = str(urlHandlers.UHConfRegistrationFormSignIn.getURL(self._conf, request.url))
        if Config.getInstance().getLoginURL().startswith("https"):
            urlLogin = urlLogin.replace("http://", "https://")
        return urlLogin


class RHRegistrationFormDisplayBaseCheckProtection(RHRegistrationFormDisplayBase):
    def _checkParams(self, params):
        RHRegistrationFormDisplayBase._checkParams(self, params)
        self._registrant = None
        self._registrant_id = params.get('registrantId', None)
        self._authkey = None
        if self._registrant_id:
            self._registrant = self._conf.getRegistrantById(self._registrant_id)
            if self._registrant is None:
                raise NotFoundError(_("The registrant with id {} does not exist or has been deleted")
                                    .format(self._registrant_id))
            self._authkey = params.get('authkey', '')
            if self._registrant.getRandomId() != self._authkey or self._authkey == '':
                raise AccessError()
        elif session.user:
            self._registrant = session.user.getRegistrantById(self._conf.getId())

    def _checkProtection(self):
        RHRegistrationFormDisplayBase._checkProtection(self)
        if self._regForm.inRegistrationPeriod() and self._regForm.isMandatoryAccount() and self._getUser() is None:
            self._redirect(self._getLoginURL())
            self._doProcess = False


class RHRegistrationFormDisplay(RHRegistrationFormDisplayBaseCheckProtection):
    _uh = urlHandlers.UHConfRegistrationFormDisplay

    def _processIfActive(self):
        if (self._conf.getRegistrationForm().isFull()
            or not self._conf.getRegistrationForm().inRegistrationPeriod()
            or (session.user and session.user.isRegisteredInConf(self._conf))):
            return redirect(url_for('event.confRegistrationFormDisplay', self._conf))
        return registrationForm.WPRegistrationFormDisplay(self, self._conf).display()


class RHRegistrationFormCreation(RHRegistrationFormDisplayBaseCheckProtection):
    _uh = urlHandlers.UHConfRegistrationFormDisplay

    def _checkParams(self, params):
        RHBaseRegistrationForm._checkParams(self, params)
        self._regForm = self._conf.getRegistrationForm()
        # SESSIONS
        sessionForm = self._regForm.getSessionsForm()
        sessions = sessionForm.getSessionsFromParams(params)
        params["sessions"] = sessions
        # ACCMMODATION
        params["accommodationType"] = self._regForm.getAccommodationForm().getAccommodationTypeById(params.get("accommodation_type", ""))
        # SOCIAL EVENTS
        socialEventIds = self._normaliseListParam(params.get("socialEvents", []))
        se = []
        for id in socialEventIds:
            se.append(self._regForm.getSocialEventForm().getSocialEventById(id))
        params["socialEvents"] = se

    def logCreation(self,registrant):
        logInfo = {}
        logInfo["subject"] =  _("Created new registrant: %s")%registrant.getFullName()
        self._conf.getLogHandler().logAction(logInfo, log.ModuleNames.REGISTRATION)

    def _process(self):
        canManageRegistration = self._conf.canManageRegistration(self._getUser())
        if not canManageRegistration and not self._regForm.isActivated():
            p = registrationForm.WPRegFormInactive(self, self._conf)
            return p.display()
        params = self._getRequestParams()
        email = self._regForm.getPersonalData().getValueFromParams(params, 'email')
        if email is None:
            raise FormValuesError(_("An email address has to be set in order to make the registration in the event."))
        elif not validMail(email, False):
            raise FormValuesError(_("The given email address is not valid."))
        matchedUsers = AvatarHolder().match({"email": email}, exact=1)
        if matchedUsers:
            user = matchedUsers[0]
        else:
            user = None
        # Check if the user can register
        if not params.get('picture_uri',"") and not(user and user.getPicture()): # check Picture
            raise FormValuesError(_("A picture is required to register."))

        if not canManageRegistration:  # normal user registering. Managers can.
            if self._conf.getRegistrationForm().isFull():
                self._redirect(urlHandlers.UHConfRegistrationFormDisplay.getURL(self._conf))
                return
            elif not self._conf.getRegistrationForm().inRegistrationPeriod():
                flash(_("Your registration couldn't be recorded."), 'error')
                return redirect(url_for('event.confRegistrationFormDisplay', self._conf))
        if user is None:
            if self._conf.hasRegistrantByEmail(email):
                raise FormValuesError("There is already a user with the email \"%s\". Please choose another one" % email)
        else:
            if params.get('existingId', '')=='None': # add new clean
                raise FormValuesError(_("There is already a user \'{0}\' with the given email \'{1}\'.").format(user.getFullName(), email))
            if params.get('existingId', None) and user.getId()<>params.get('existingId'): # add new existing
                raise FormValuesError(_("The registration email \'{0}\' does not match the user email.").format(email))
            if not params.get('existingId', None) and email<>self.getAW().getUser().getEmail(): # user register
                raise FormValuesError(_("The registration email \'{0}\' does not match the user email.").format(email))

            if user.isRegisteredInConf(self._conf) or self._conf.hasRegistrantByEmail(email):
                if canManageRegistration:
                    raise FormValuesError("There is already a user with the email \"%s\". Please choose another one" % email)
                else:
                    raise FormValuesError("You have already registered with the email address \"%s\". If you need to modify your registration, please contact the managers of the conference." % email)

        rp = registration.Registrant()
        self._conf.addRegistrant(rp, user)
        rp.setValues(self._getRequestParams(), user)

        if user is not None:
            user.addRegistrant(rp)
            rp.setAvatar(user)
        elif(self._conf.getRegistrationForm().isMandatoryAccount()): #account mandatory for registration
            rp.createAvatar() # create an account for this registration

        self.logCreation(rp)
        notify_registration_confirmation(self._conf, rp)

        if canManageRegistration and user != self._getUser():
            self._redirect(RHRegistrantListModif._uh.getURL(self._conf))
        else:
            params = {}
            if not rp.doPay() or not payment_event_settings.get(self._conf, 'enabled'):
                flash(_(u"Your registration has been recorded successfully."), 'success')
            if not session.user:
                params.update(rp.getLocator(), **{'authkey': rp.getRandomId()})
            else:
                params.update(self._conf.getLocator())
            return redirect(url_for('event.confRegistrationFormDisplay', **params))


class RHRegistrationForm(RHRegistrationFormDisplayBase):
    _uh = urlHandlers.UHConfRegistrationForm

    def _checkParams(self, params):
        RHRegistrationFormDisplayBase._checkParams(self, params)
        self._registrant = None
        self._registrant_id = params.get('registrantId', None)
        self._authkey = None
        if self._registrant_id:
            self._registrant = self._conf.getRegistrantById(self._registrant_id)
            if self._registrant is None:
                raise NotFoundError(_("The registrant with id {} does not exist or has been deleted")
                                    .format(self._registrant_id))
            self._authkey = params.get('authkey', '')
            if self._registrant.getRandomId() != self._authkey or self._authkey == '':
                raise AccessError()
        elif session.user:
            self._registrant = session.user.getRegistrantById(self._conf.getId())

    def _processIfActive(self):
        p = registrationForm.WPRegistrationForm(self, self._conf)
        return p.display(registrant=self._registrant, registrant_id=self._registrant_id, authkey=self._authkey)


class RHRegistrationFormRegistrantBase(RHRegistrationForm):
    def _checkParams(self, params):
        RHRegistrationForm._checkParams(self, params)
        if self._registrant is None:
            if not session.user and not params.get('registrantId', None):
                raise MaKaCError(_("registrant id not set"))
            raise NotFoundError(_("You are not registered or your registration has been deleted"))


class RHConferenceTicketPDF(RHRegistrationFormDisplayBaseCheckProtection):
    def _process(self):
        set_best_lang()  # prevents from having a _LazyString when generating a pdf without session.lang set
        filename = "{0}-Ticket.pdf".format(self._target.getTitle())
        pdf = TicketToPDF(self._target, self._registrant)
        return send_file(filename, StringIO(pdf.getPDFBin()), 'PDF')


class RHRegistrationFormModify(RHRegistrationFormDisplayBaseCheckProtection):
    _uh = urlHandlers.UHConfRegistrationFormDisplay

    def _checkParams(self, params):
        RHRegistrationFormDisplayBase._checkParams(self, params)
        if 'registrantId' in params and session.user:
            if self._conf.canManageRegistration(session.user):
                self._registrant = self._conf.getRegistrantById(params['registrantId'])
        else:
            self._registrant = session.user.getRegistrantById(self._conf.getId()) if session.user else None

    def _processIfActive(self):
        if not self._registrant:
            return redirect(url_for('event.confRegistrationFormDisplay-display', self._conf))
        if not self._regForm.inModificationPeriod():
            flash(_("The modification period is over."), 'error')
            return redirect(url_for('event.confRegistrationFormDisplay', self._conf))
        return registrationForm.WPRegistrationFormModify(self, self._conf).display()


class RHRegistrationFormPerformModify(RHRegistrationFormCreation):
    _uh = urlHandlers.UHConfRegistrationFormModify

    def _checkParams(self, params):
        RHRegistrationFormCreation._checkParams(self,params)
        self.registrantId = params.get('regId',None)

    def _process(self):
        if self.registrantId or (self._getUser() is not None and self._getUser().isRegisteredInConf(self._conf)):
            if not self._getUser().isAdmin() and not self._conf.getRegistrationForm().inRegistrationPeriod() and not self._conf.getRegistrationForm().inModificationPeriod():
                flash(_("Your modification could not be recorded since the modification period is over."), 'error')
                return redirect(url_for('event.confRegistrationFormDisplay', self._conf))
            else:
                rp = self._conf.getRegistrantById(self.registrantId) \
                    if self.registrantId \
                    else \
                    self._getUser().getRegistrantById(self._conf.getId())
                # check if the email is being changed by another one that already exists
                # if self._getRequestParams().get("email", "") != rp.getEmail() and self._conf.hasRegistrantByEmail(self._getRequestParams().get("email", "")):
                #     raise FormValuesError(_("There is already a user with the email \"%s\". Please choose another one") % self._getRequestParams().get("email", "--no email--"))
                paraEmail = self._regForm.getPersonalData().getValueFromParams(self._getRequestParams(), 'email')
                if paraEmail != rp.getEmail():
                    raise FormValuesError(_("Registration email \'{0}\' is different than User email \'{1}\'.").format(paraEmail, rp.getEmail()))
                if self.registrantId:
                    rp.setValues(self._getRequestParams(), rp.getAvatar())
                else:
                    rp.setValues(self._getRequestParams(), self._getUser())

                if not (self._getUser().isAdmin() or self._conf.canUserModify(self._getUser())):
                    notify_registration_modification(self._conf, rp)
                flash(_(u"Your registration has been modified successfully."), 'success')
                if rp.doPay():
                    self._redirect(urlHandlers.UHConfRegistrationFormCreationDone.getURL(rp))
                else:
                    if self.registrantId:
                        url = urlHandlers.UHRegistrantModification.getURL(rp)
                    else:
                        url = urlHandlers.UHConfRegistrationForm.getURL(self._conf)
                    self._redirect(url)
        else:
            self._redirect(urlHandlers.UHConfRegistrationForm.getURL(self._conf))


class RHRegistrationFormConditions(RHRegistrationFormDisplayBaseCheckProtection):

    def _process(self):
        p = registrationForm.WPRegistrationFormConditions(self, self._conf)
        return p.display()


class RHRegistrationFormUserData(RHRegistrationFormDisplayBaseCheckProtection):

    def _checkParams(self, params):
        RHRegistrationFormDisplayBaseCheckProtection._checkParams(self,params)
        self._existingId = params.get('existingId', None) # Case: Add new registrant from existing
        if 'regId' in params: # Case: Modify registrant from registrants list
            registrantId = params.get('regId')
            self._registrant = self._conf.getRegistrantById(registrantId)

    def _checkProtection(self):
        RHBaseRegistrationForm._checkProtection(self)

    def _process(self):
        if self._existingId: # case registration created by manager
            self._registrant = None
            user = None if self._existingId=='None' else AvatarHolder().getById(self._existingId)
        else:
            user = self._aw.getUser()

        newRegistration = False
        # look for a previous registration of user
        if not(self._registrant) and user and len(user.getRegistrantList())>0:
            self._registrant = sorted(user.getRegistrantList(),key=lambda r: r.getRegistrationDate(),reverse=True)[0]
            newRegistration = True

        reg_data = {}
        if self._registrant is not None: # data from Registrant
            reg_data = self._registrant.fossilize()

            # take personal data from Avatar and map fields with previous registration
            if newRegistration:
                reg_data['miscellaneousGroupList'] = self.updateMiscellaneousGroupList(reg_data['miscellaneousGroupList'])
                targets = dict( (item['title'], item['responseItems']) for item in reg_data['miscellaneousGroupList'] )
                from indico.util.fossilize import fossilize
                src = fossilize(section for section in self._conf.getRegistrationForm().getSortedForms() if section.isEnabled())
                items = dict( (item['title'], item['items']) for item in src if "items" in item )
                if 'Personal Data' in targets: # match personal data field names
                    section_items = targets['Personal Data']
                    for item in section_items:
                        s = section_items[item]
                        for i in items['Personal Data']:
                            if i['name'] == s['name']:
                                s['HTMLName'] = i['values']['HTMLName']
                                break
            if(reg_data['sessionList']==[]):
                for _ in range(len(self._conf.getSessionList())):
                    reg_data['sessionList'].append({})
            if ('regId' not in self._reqParams) and ('existingId' not in self._reqParams) and \
                (self._registrant.getAvatar()==self.getAW().getUser()): # case user modify his registration
                reg_data["hideContactType"] = True
        elif user is not None:
            personalData = self._regForm.getPersonalData()
            reg_data['avatar'] = personalData.getFormValuesFromAvatar(user)
            reg_data['avatar'].setdefault('sessionList', [{}, {}])
            if ('existingId' not in self._reqParams): # case New Registration by user
                reg_data['avatar']['hideContactType'] = True
        else:
            #reg_data['avatar'] = {'sessionList': [{}, {}]}
            sl = []
            for ses in self._regForm.getSessionsForm().getSessionList():
                sl.append({'_fossil': 'regFormSessionsForm', '_type': 'RegistrantSession', 'id': ses.getId()})
            reg_data['avatar'] = {'sessionList': sl}

        # Disable modification of Representation Type for approved participants, Managers still can modify
        if not newRegistration and self._registrant and self._registrant.isApproved() \
            and ('regId' not in self._reqParams):
            reg_data["representationReadOnly"] = True
        else:
            reg_data["representationReadOnly"] = False

        return json.dumps(reg_data)


    def updateMiscellaneousGroupList(self, miscellaneousGroupList):
        # keep only Personal Data in miscellaneousGroupList
        # fill personal data from avatar
        newGrp = []
        for pd in miscellaneousGroupList:
            if pd['title']=='Personal Data':
                newGrp.append(pd)
                break
        if not pd:
            return miscellaneousGroupList
        avatar = self._registrant.getAvatar()
        import datetime
        for item in pd['responseItems'].values():
            if item['name']=='title':
                item['value'] = avatar.getTitle()
            elif item['name']=='gender':
                item['value'] = avatar.getGender()
            elif item['name']=='firstName':
                item['value'] = avatar.getFirstName()
            elif item['name']=='surname':
                item['value'] = avatar.getSurName()
            elif item['name']=='institution':
                item['value'] = avatar.getOrganisation()
            elif item['name']=='birthDate':
                birthDate = datetime.datetime.strptime(avatar.getBirthDate(),'%d/%m/%Y') if avatar.getBirthDate() and isinstance(avatar.getBirthDate(),basestring) \
                            else avatar.getBirthDate()
                item['value'] = birthDate if avatar.getBirthDate() else None
            elif item['name']=='address':
                item['value'] = avatar.getAddress()
            elif item['name']=='phone':
                item['value'] = avatar.getPhone()
            elif item['name']=='fax':
                item['value'] = avatar.getFax()
            elif item['name']=='email':
                item['value'] = avatar.getEmail()
            elif item['name']=='passportId':
                item['value'] = avatar.getPassportId()
            elif item['name']=='passportOrigin':
                item['value'] = avatar.getPassportOrigin()
            elif item['name']=='passportExpire':
                passportExpire = datetime.datetime.strptime(avatar.getPassportExpire(),'%d/%m/%Y') if avatar.getPassportExpire() and isinstance(avatar.getPassportExpire(),basestring) \
                            else avatar.getPassportExpire()
                item['value'] = passportExpire if avatar.getPassportExpire() else None
            elif item['name']=='picture':
                item['value']['path'] = avatar.getPictureUrl()
                item['value']['name'] = avatar.getPicture().getFileName() if avatar.getPicture() else None
        return newGrp


class RHRegistrationFormReprLists(RHRegistrationFormDisplayBaseCheckProtection):

    def _checkParams(self, params):
        RHRegistrationFormDisplayBaseCheckProtection._checkParams(self,params)

    def _checkProtection(self):
        RHBaseRegistrationForm._checkProtection(self)

    def _process(self):
        result = dict()
        result["countryList"]=RHRegistrationFormReprLists.getCountryList(self._conf)
        #result["organizationList"]=RHRegistrationFormReprLists.getOrganizationList(self._conf)
        result["representationTypes"]=RHRegistrationFormReprLists.getRepresentationTypes(self._conf)
        payload = json.dumps(result)
        return payload

    @staticmethod
    def getCountryList(conf=None):
        from MaKaC.webinterface.common.countries import CountryHolder
        list = {}
        for country in CountryHolder().getCountryList():
            list[country]= country
        return list

    @staticmethod
    def scanParent(_conf, _list, _type=None):
        if not _conf:
            return _list['default']

        parent = _conf.getOwner()
        while parent:
            if parent.getEmeetsClientCode() in _list:
                try:
                    orgList = _list[parent.getEmeetsClientCode()]
                    if not isinstance(orgList, dict): # ECOSOC list
                        return orgList
                    else: # other lists
                        return orgList
                        # return orgList[_type] : not used because all categories are merged in one
                except KeyError:
                    return _list['default']
            else:
                parent = parent.getOwner()
        return _list['default']

    @staticmethod
    def getOrganizationList(listType, conf=None):
#        from MaKaC.webinterface.listmapping import UNListMapping
        listClass = RHRegistrationFormReprLists.scanParent(conf, UNListMapping.orgListsMapping, listType)
        if isinstance(listClass, dict):
            return listClass
        return listClass.getInstance().getList()

    @staticmethod
    def getRepresentationTypes(conf=None):
        # from MaKaC.webinterface.listmapping import UNListMapping
        list = RHRegistrationFormReprLists.scanParent(conf,UNListMapping.reprTypesMapping)
        return list
