# This file is part of Indico.
# Copyright (C) 2002 - 2015 European Organization for Nuclear Research (CERN).
#
# Indico is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Indico is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Indico; if not, see <http://www.gnu.org/licenses/>.

from MaKaC.conference import ConferenceHolder

from indico.core.fossils.registration import IRegFormRegistrantBasicFossil, IRegFormRegistrantFullFossil
from indico.modules.payment import event_settings as payment_event_settings

from indico.modules.payment.models.transactions import TransactionAction
from indico.modules.payment.util import register_transaction
from indico.web.http_api.hooks.base import HTTPAPIHook, DataFetcher
from indico.web.http_api.hooks.event import EventBaseHook
from indico.web.http_api.util import get_query_parameter
from indico.web.http_api.responses import HTTPAPIError
import MaKaC.webinterface.urlHandlers as urlHandlers

from indico.util.date_time import format_datetime
from indico.util.fossilize import fossilize
import json

@HTTPAPIHook.register
class SetPaidHook(EventBaseHook):
    PREFIX = "api"
    RE = r'(?P<event>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)/pay'
    METHOD_NAME = 'api_pay'
    NO_CACHE = True
    COMMIT = True
    HTTP_POST = True

    def _getParams(self):
        super(SetPaidHook, self)._getParams()
        self.auth_key = get_query_parameter(self._queryParams, ["auth_key"])
        self.is_paid = get_query_parameter(self._queryParams, ["is_paid"]) == "yes"
        registrant_id = self._pathParams["registrant_id"]
        self._conf = ConferenceHolder().getById(self._pathParams['event'])
        self._registrant = self._conf.getRegistrantById(registrant_id)
        if not payment_event_settings.get(self._conf, 'enabled'):
            raise HTTPAPIError('E-payment is not enabled')

    def _hasAccess(self, aw):
        return self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw)

    def api_pay(self, aw):
        action = TransactionAction.complete if self._isPayed == '1' else TransactionAction.cancel
        register_transaction(registrant=self._registrant,
                             amount=self._registrant.getTotal(),
                             currency=self._registrant.getCurrency(),
                             action=action)
        return {
            "paid": self._registrant.getPayed(),
            "amount_paid": self._registrant.getTotal()
        }


class BaseCheckInHook(EventBaseHook):
    PREFIX = "api"
    METHOD_NAME = 'api_checkin'
    NO_CACHE = True
    COMMIT = True
    HTTP_POST = True

    def _getParams(self):
        super(BaseCheckInHook, self)._getParams()
        self._check_in = get_query_parameter(self._queryParams, ["checked_in"]) == "yes"
        registrant_id = self._pathParams["registrant_id"]
        self._conf = ConferenceHolder().getById(self._pathParams['event'])
        self._object = self._conf.getRegistrantById(registrant_id)

    def _hasAccess(self, aw):
        return (self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw))

    def api_checkin(self, aw):
        if self._object:
            self._object.setCheckedIn(self._check_in)
            checkin_date = format_datetime(self._object.getAdjustedCheckInDate(), format="short") if self._check_in else None

            return {
                "checked_in": self._check_in,
                "checkin_date": checkin_date if self._check_in else None
            }
        else:
            return {"status": "no session has been found"}



@HTTPAPIHook.register
class CheckInHook(BaseCheckInHook):
    RE = r'(?P<event>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)/checkin'


@HTTPAPIHook.register
class SessionCheckInHook(BaseCheckInHook):
    RE = r'(?P<event>[\w\s]+)/session/(?P<session_id>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)/checkin'

    def _getParams(self):
        super(SessionCheckInHook, self)._getParams()
        session_id = self._pathParams["session_id"]
        self._object = self._object.getSession(session_id)


@HTTPAPIHook.register
class UpdatePictureHook(EventBaseHook):
    PREFIX = "api"
    RE = r'(?P<event>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)/updatepicture'
    METHOD_NAME = 'api_update_picture'
    NO_CACHE = True
    COMMIT = True
    HTTP_POST = True

    def _getParams(self):
        super(UpdatePictureHook, self)._getParams()
        self._picture_payload = get_query_parameter(self._queryParams, ["picture_uri"])
        registrant_id = self._pathParams["registrant_id"]
        self._conf = ConferenceHolder().getById(self._pathParams['event'])
        self._registrant = self._conf.getRegistrantById(registrant_id)

    def _hasAccess(self, aw):
        return (self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw))

    def api_update_picture(self, aw):
        try:
            from indico.util.data_uri import DataURI
            uri = DataURI(self._picture_payload)
            path = self._registrant.getPicture().getFilePath()
            data = uri.data
            _file = open(path,'w')
            _file.write(data)
            _file.close()
            return {"status": "true"}
        except:
            return {"status": "false"}


def ascii_encode_dict(data):
    ascii_encode = lambda x: x.encode('ascii')
    return dict(map(ascii_encode, pair) for pair in data.items())


@HTTPAPIHook.register
class UpdatePassportHook(EventBaseHook):
    PREFIX = "api"
    RE = r'(?P<event>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)/updatepassport'
    METHOD_NAME = 'api_update_passport'
    NO_CACHE = True
    COMMIT = True
    HTTP_POST = True

    def _getParams(self):
        super(UpdatePassportHook, self)._getParams()
        raw_passport_info = str(get_query_parameter(self._queryParams, ["passport_info"]))
        self._passport_info = json.loads(raw_passport_info,object_hook=ascii_encode_dict)
        registrant_id = self._pathParams["registrant_id"]
        self._conf = ConferenceHolder().getById(self._pathParams['event'])
        self._registrant = self._conf.getRegistrantById(registrant_id)

    def _hasAccess(self, aw):
        return (self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw))

    def api_update_passport(self, aw):
        try:
            self._registrant.updateFromPassportScan(self._passport_info)
            return {"status": "true"}
        except Exception as e:
            return {"status": "false"}


class BaseRegistrantHook(EventBaseHook):
    METHOD_NAME = 'export_registrant'
    NO_CACHE = True
    DEFAULT_DETAIL = 'basic'

    def _getParams(self):
        super(BaseRegistrantHook, self)._getParams()
        self.auth_key = get_query_parameter(self._queryParams, ["auth_key"])
        self._conf = ConferenceHolder().getById(self._pathParams['event'])
        registrant_id = self._pathParams["registrant_id"]
        self._registrant = self._conf.getRegistrantById(registrant_id)

    def _hasAccess(self, aw):
        return self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw)

    def export_registrant(self, aw, bysession=None):


        expInt = RegistrantFetcher(aw, self)
        result =  expInt.registrant()
        if bysession and not self._registrant.getSession(bysession):
            result["canEnter"] = False
            result["canEnterMessage"] = "The registrant is not approved for this session"
        return result


class RegistrantFetcher(DataFetcher):
    DETAIL_INTERFACES = {
        'basic': IRegFormRegistrantBasicFossil,
        'full': IRegFormRegistrantFullFossil
    }

    def __init__(self, aw, hook):
        super(RegistrantFetcher, self).__init__(aw, hook)
        self._registrant = hook._registrant
        self._conf = hook._conf
        self._detail = hook._detail

    def _makeFossil(self):
        iface = self.DETAIL_INTERFACES.get(self._detail)
        if iface is None:
            raise HTTPAPIError('Invalid detail level: %s' % self._detail, 400)

        return fossilize(self._registrant, iface)

    def registrant(self):
        result = self._makeFossil()
        if self._detail == 'basic':
            regForm = self._conf.getRegistrationForm()
            personalData = regForm.getPersonalData().getRegistrantValues(self._registrant)
            personalData['picture'] = self._registrant.getPictureUrl()
            personalData['passportId'] = self._registrant.getPassportId()
            personalData['passportExpire'] = self._registrant.getPassportExpire().strftime('%d/%m/%y')
            personalData['passportOrigin'] = self._registrant.getPassportOrigin()
            result["personal_data"] = personalData

            result["canEnter"],result["canEnterMessage"] = self._registrant.canEnterPremises()
        return result


@HTTPAPIHook.register
class RegistrantHook(BaseRegistrantHook):
    RE = r'(?P<event>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)'


@HTTPAPIHook.register
class RegistrantBySessionHook(BaseRegistrantHook):
    RE = r'(?P<event>[\w\s]+)/session/(?P<session>[\w\s]+)/registrant/(?P<registrant_id>[\w\s]+)'

    def _getParams(self):
        super(RegistrantBySessionHook, self)._getParams()
        self._sessionId = self._pathParams['session']

    def export_registrant(self, aw,bysession=None):
        return super(RegistrantBySessionHook, self).export_registrant(aw,self._sessionId)


class BaseRegistrantsHook(EventBaseHook):
    METHOD_NAME = 'export_registrants'
    NO_CACHE = True

    def _getParams(self):
        super(BaseRegistrantsHook, self)._getParams()
        self._conf_id = self._pathParams['event']
        self._conf = ConferenceHolder().getById(self._conf_id)
        self._registrants = self._conf.getRegistrantsList()

    def _hasAccess(self, aw):
        return self._conf.canManageRegistration(aw.getUser()) or self._conf.canModify(aw)

    def export_registrants(self, aw,bysession=None):

        registrant_list = []
        for registrant in self._registrants:
            skip = False

            if bysession:
                sess = registrant.getSession(bysession)
                if sess is None:
                    skip = True
                else:
                    checkedIn = sess.isCheckedIn()
            else:
                checkedIn = registrant.isCheckedIn(),

            if not skip:
                reg = {
                    "registrant_id": registrant.getId(),
                    "checked_in": checkedIn,
                    "full_name": registrant.getFullName(title=True, firstNameFirst=True),
                    "checkin_secret": registrant.getCheckInUUID(),
                }
                if bysession:
                    reg["session"]=bysession
                regForm = self._conf.getRegistrationForm()
                reg["personal_data"] = regForm.getPersonalData().getRegistrantValues(registrant)
                reg["personal_data"]["picture"]=registrant.getPictureUrl()
                registrant_list.append(reg)
        return {"registrants": registrant_list}


@HTTPAPIHook.register
class RegistrantsHook(BaseRegistrantsHook):
    RE = r'(?P<event>[\w\s]+)/registrants'


@HTTPAPIHook.register
class RegistrantsBySessionHook(BaseRegistrantsHook):
    RE = r'(?P<event>[\w\s]+)/session/(?P<session>[\w\s]+)/registrants'

    def _getParams(self):
        super(RegistrantsBySessionHook, self)._getParams()
        self._sessionId = self._pathParams['session']

    def export_registrants(self, aw,bysession=None):
        return super(RegistrantsBySessionHook, self).export_registrants(aw,self._sessionId)